@@ -1,23 +1,24 @@
 """WebSocket protocol versions 13 and 8."""
 
-import base64
-import binascii
+import asyncio
 import collections
-import hashlib
 import json
 import random
+import re
 import sys
+import zlib
 from enum import IntEnum
 from struct import Struct
+from typing import Any, Callable, List, Optional, Tuple, Union
 
-from . import hdrs
-from .helpers import NO_EXTENSIONS, noop
-from .http_exceptions import HttpBadRequest, HttpProcessingError
+from .base_protocol import BaseProtocol
+from .helpers import NO_EXTENSIONS
 from .log import ws_logger
+from .streams import DataQueue
 
 __all__ = ('WS_CLOSED_MESSAGE', 'WS_CLOSING_MESSAGE', 'WS_KEY',
-           'WebSocketReader', 'WebSocketWriter', 'do_handshake',
-           'WSMessage', 'WebSocketError', 'WSMsgType', 'WSCloseCode')
+           'WebSocketReader', 'WebSocketWriter', 'WSMessage',
+           'WebSocketError', 'WSMsgType', 'WSCloseCode')
 
 
 class WSCloseCode(IntEnum):
@@ -81,17 +82,14 @@ _WSMessageBase = collections.namedtuple('_WSMessageBase',
 
 class WSMessage(_WSMessageBase):
 
-    def json(self, *, loads=json.loads):
+    def json(self, *,  # type: ignore
+             loads: Callable[[Any], Any]=json.loads) -> Any:
         """Return parsed JSON data.
 
         .. versionadded:: 0.22
         """
         return loads(self.data)
 
-    @property
-    def tp(self):
-        return self.type
-
 
 WS_CLOSED_MESSAGE = WSMessage(WSMsgType.CLOSED, None, None)
 WS_CLOSING_MESSAGE = WSMessage(WSMsgType.CLOSING, None, None)
@@ -100,21 +98,30 @@ WS_CLOSING_MESSAGE = WSMessage(WSMsgType.CLOSING, None, None)
 class WebSocketError(Exception):
     """WebSocket protocol parser error."""
 
-    def __init__(self, code, message):
+    def __init__(self, code: int, message: str) -> None:
         self.code = code
         super().__init__(message)
 
 
+class WSHandshakeError(Exception):
+    """WebSocket protocol handshake error."""
+
+
 native_byteorder = sys.byteorder
 
 
-def _websocket_mask_python(mask, data):
+# Used by _websocket_mask_python
+_XOR_TABLE = [bytes(a ^ b for a in range(256)) for b in range(256)]
+
+
+def _websocket_mask_python(mask: bytes, data: bytearray) -> None:
     """Websocket masking function.
 
-    `mask` is a `bytes` object of length 4; `data` is a `bytes` object
-    of any length.  Returns a `bytes` object of the same length as
-    `data` with the mask applied as specified in section 5.3 of RFC
-    6455.
+    `mask` is a `bytes` object of length 4; `data` is a `bytearray`
+    object of any length. The contents of `data` are masked with `mask`,
+    as specified in section 5.3 of RFC 6455.
+
+    Note that this function mutates the `data` argument.
 
     This pure-python implementation may be replaced by an optimized
     version when available.
@@ -122,25 +129,107 @@ def _websocket_mask_python(mask, data):
     """
     assert isinstance(data, bytearray), data
     assert len(mask) == 4, mask
-    datalen = len(data)
-    if datalen == 0:
-        # everything work without this, but may be changed later in Python.
-        return bytearray()
-    data = int.from_bytes(data, native_byteorder)
-    mask = int.from_bytes(mask * (datalen // 4) + mask[: datalen % 4],
-                          native_byteorder)
-    return (data ^ mask).to_bytes(datalen, native_byteorder)
+
+    if data:
+        a, b, c, d = (_XOR_TABLE[n] for n in mask)
+        data[::4] = data[::4].translate(a)
+        data[1::4] = data[1::4].translate(b)
+        data[2::4] = data[2::4].translate(c)
+        data[3::4] = data[3::4].translate(d)
 
 
-if NO_EXTENSIONS:
+if NO_EXTENSIONS:  # pragma: no cover
     _websocket_mask = _websocket_mask_python
 else:
     try:
-        from ._websocket import _websocket_mask_cython
+        from ._websocket import _websocket_mask_cython  # type: ignore
         _websocket_mask = _websocket_mask_cython
     except ImportError:  # pragma: no cover
         _websocket_mask = _websocket_mask_python
 
+_WS_DEFLATE_TRAILING = bytes([0x00, 0x00, 0xff, 0xff])
+
+
+_WS_EXT_RE = re.compile(r'^(?:;\s*(?:'
+                        r'(server_no_context_takeover)|'
+                        r'(client_no_context_takeover)|'
+                        r'(server_max_window_bits(?:=(\d+))?)|'
+                        r'(client_max_window_bits(?:=(\d+))?)))*$')
+
+_WS_EXT_RE_SPLIT = re.compile(r'permessage-deflate([^,]+)?')
+
+
+def ws_ext_parse(extstr: str, isserver: bool=False) -> Tuple[int, bool]:
+    if not extstr:
+        return 0, False
+
+    compress = 0
+    notakeover = False
+    for ext in _WS_EXT_RE_SPLIT.finditer(extstr):
+        defext = ext.group(1)
+        # Return compress = 15 when get `permessage-deflate`
+        if not defext:
+            compress = 15
+            break
+        match = _WS_EXT_RE.match(defext)
+        if match:
+            compress = 15
+            if isserver:
+                # Server never fail to detect compress handshake.
+                # Server does not need to send max wbit to client
+                if match.group(4):
+                    compress = int(match.group(4))
+                    # Group3 must match if group4 matches
+                    # Compress wbit 8 does not support in zlib
+                    # If compress level not support,
+                    # CONTINUE to next extension
+                    if compress > 15 or compress < 9:
+                        compress = 0
+                        continue
+                if match.group(1):
+                    notakeover = True
+                # Ignore regex group 5 & 6 for client_max_window_bits
+                break
+            else:
+                if match.group(6):
+                    compress = int(match.group(6))
+                    # Group5 must match if group6 matches
+                    # Compress wbit 8 does not support in zlib
+                    # If compress level not support,
+                    # FAIL the parse progress
+                    if compress > 15 or compress < 9:
+                        raise WSHandshakeError('Invalid window size')
+                if match.group(2):
+                    notakeover = True
+                # Ignore regex group 5 & 6 for client_max_window_bits
+                break
+        # Return Fail if client side and not match
+        elif not isserver:
+            raise WSHandshakeError('Extension for deflate not supported' +
+                                   ext.group(1))
+
+    return compress, notakeover
+
+
+def ws_ext_gen(compress: int=15, isserver: bool=False,
+               server_notakeover: bool=False) -> str:
+    # client_notakeover=False not used for server
+    # compress wbit 8 does not support in zlib
+    if compress < 9 or compress > 15:
+        raise ValueError('Compress wbits must between 9 and 15, '
+                         'zlib does not support wbits=8')
+    enabledext = ['permessage-deflate']
+    if not isserver:
+        enabledext.append('client_max_window_bits')
+
+    if compress < 15:
+        enabledext.append('server_max_window_bits=' + str(compress))
+    if server_notakeover:
+        enabledext.append('server_no_context_takeover')
+    # if client_notakeover:
+    #     enabledext.append('client_no_context_takeover')
+    return '; '.join(enabledext)
+
 
 class WSParserState(IntEnum):
     READ_HEADER = 1
@@ -151,28 +240,33 @@ class WSParserState(IntEnum):
 
 class WebSocketReader:
 
-    def __init__(self, queue):
+    def __init__(self, queue: DataQueue[WSMessage],
+                 max_msg_size: int, compress: bool=True) -> None:
         self.queue = queue
+        self._max_msg_size = max_msg_size
 
-        self._exc = None
-        self._partial = []
+        self._exc = None  # type: Optional[BaseException]
+        self._partial = bytearray()
         self._state = WSParserState.READ_HEADER
 
-        self._opcode = None
+        self._opcode = None  # type: Optional[int]
         self._frame_fin = False
-        self._frame_opcode = None
+        self._frame_opcode = None  # type: Optional[int]
         self._frame_payload = bytearray()
 
         self._tail = b''
         self._has_mask = False
-        self._frame_mask = None
+        self._frame_mask = None  # type: Optional[bytes]
         self._payload_length = 0
         self._payload_length_flag = 0
+        self._compressed = None  # type: Optional[bool]
+        self._decompressobj = None  # type: Any  # zlib.decompressobj actually
+        self._compress = compress
 
-    def feed_eof(self):
+    def feed_eof(self) -> None:
         self.queue.feed_eof()
 
-    def feed_data(self, data):
+    def feed_data(self, data: bytes) -> Tuple[bool, bytes]:
         if self._exc:
             return True, data
 
@@ -183,8 +277,10 @@ class WebSocketReader:
             self.queue.set_exception(exc)
             return True, b''
 
-    def _feed_data(self, data):
-        for fin, opcode, payload in self.parse_frame(data):
+    def _feed_data(self, data: bytes) -> Tuple[bool, bytes]:
+        for fin, opcode, payload, compressed in self.parse_frame(data):
+            if compressed and not self._decompressobj:
+                self._decompressobj = zlib.decompressobj(wbits=-zlib.MAX_WBITS)
             if opcode == WSMsgType.CLOSE:
                 if len(payload) >= 2:
                     close_code = UNPACK_CLOSE_CODE(payload[:2])[0]
@@ -219,18 +315,23 @@ class WebSocketReader:
                     WSMessage(WSMsgType.PONG, payload, ''), len(payload))
 
             elif opcode not in (
-                    WSMsgType.TEXT, WSMsgType.BINARY) and not self._opcode:
+                    WSMsgType.TEXT, WSMsgType.BINARY) and self._opcode is None:
                 raise WebSocketError(
                     WSCloseCode.PROTOCOL_ERROR,
                     "Unexpected opcode={!r}".format(opcode))
             else:
                 # load text/binary
-
                 if not fin:
                     # got partial frame payload
                     if opcode != WSMsgType.CONTINUATION:
                         self._opcode = opcode
-                    self._partial.append(payload)
+                    self._partial.extend(payload)
+                    if (self._max_msg_size and
+                            len(self._partial) >= self._max_msg_size):
+                        raise WebSocketError(
+                            WSCloseCode.MESSAGE_TOO_BIG,
+                            "Message size {} exceeds limit {}".format(
+                                len(self._partial), self._max_msg_size))
                 else:
                     # previous frame was non finished
                     # we should get continuation opcode
@@ -242,13 +343,39 @@ class WebSocketReader:
                                 'to be zero, got {!r}'.format(opcode))
 
                     if opcode == WSMsgType.CONTINUATION:
+                        assert self._opcode is not None
                         opcode = self._opcode
+                        self._opcode = None
 
-                    self._partial.append(payload)
+                    self._partial.extend(payload)
+                    if (self._max_msg_size and
+                            len(self._partial) >= self._max_msg_size):
+                        raise WebSocketError(
+                            WSCloseCode.MESSAGE_TOO_BIG,
+                            "Message size {} exceeds limit {}".format(
+                                len(self._partial), self._max_msg_size))
+
+                    # Decompress process must to be done after all packets
+                    # received.
+                    if compressed:
+                        self._partial.extend(_WS_DEFLATE_TRAILING)
+                        payload_merged = self._decompressobj.decompress(
+                            self._partial, self._max_msg_size)
+                        if self._decompressobj.unconsumed_tail:
+                            left = len(self._decompressobj.unconsumed_tail)
+                            raise WebSocketError(
+                                WSCloseCode.MESSAGE_TOO_BIG,
+                                "Decompressed message size exceeds limit {}".
+                                format(self._max_msg_size + left,
+                                       self._max_msg_size))
+                    else:
+                        payload_merged = bytes(self._partial)
+
+                    self._partial.clear()
 
                     if opcode == WSMsgType.TEXT:
                         try:
-                            text = b''.join(self._partial).decode('utf-8')
+                            text = payload_merged.decode('utf-8')
                             self.queue.feed_data(
                                 WSMessage(WSMsgType.TEXT, text, ''), len(text))
                         except UnicodeDecodeError as exc:
@@ -256,26 +383,24 @@ class WebSocketReader:
                                 WSCloseCode.INVALID_TEXT,
                                 'Invalid UTF-8 text message') from exc
                     else:
-                        data = b''.join(self._partial)
                         self.queue.feed_data(
-                            WSMessage(WSMsgType.BINARY, data, ''), len(data))
-
-                    self._start_opcode = None
-                    self._partial.clear()
+                            WSMessage(WSMsgType.BINARY, payload_merged, ''),
+                            len(payload_merged))
 
         return False, b''
 
-    def parse_frame(self, buf, continuation=False, EMPTY=b''):
+    def parse_frame(self, buf: bytes) -> List[Tuple[bool, Optional[int],
+                                                    bytearray,
+                                                    Optional[bool]]]:
         """Return the next frame from the socket."""
         frames = []
         if self._tail:
-            buf, self._tail = self._tail + buf, EMPTY
+            buf, self._tail = self._tail + buf, b''
 
         start_pos = 0
         buf_length = len(buf)
 
         while True:
-
             # read header
             if self._state == WSParserState.READ_HEADER:
                 if buf_length - start_pos >= 2:
@@ -297,7 +422,9 @@ class WebSocketReader:
                     #    1 bit, MUST be 0 unless negotiated otherwise
                     # frame-rsv3 = %x0 ;
                     #    1 bit, MUST be 0 unless negotiated otherwise
-                    if rsv1 or rsv2 or rsv3:
+                    #
+                    # Remove rsv1 from this test for deflate development
+                    if rsv2 or rsv3 or (rsv1 and not self._compress):
                         raise WebSocketError(
                             WSCloseCode.PROTOCOL_ERROR,
                             'Received frame with non-zero reserved bits')
@@ -307,17 +434,8 @@ class WebSocketReader:
                             WSCloseCode.PROTOCOL_ERROR,
                             'Received fragmented control frame')
 
-                    continuation = not self._frame_fin
-                    if (fin == 0 and
-                            opcode == WSMsgType.CONTINUATION and
-                            not continuation):
-                        raise WebSocketError(
-                            WSCloseCode.PROTOCOL_ERROR,
-                            'Received new fragment frame with non-zero '
-                            'opcode {!r}'.format(opcode))
-
                     has_mask = (second_byte >> 7) & 1
-                    length = (second_byte) & 0x7f
+                    length = second_byte & 0x7f
 
                     # Control frames MUST have a payload
                     # length of 125 bytes or less
@@ -327,9 +445,19 @@ class WebSocketReader:
                             'Control frame payload cannot be '
                             'larger than 125 bytes')
 
-                    self._frame_fin = fin
+                    # Set compress status if last package is FIN
+                    # OR set compress status if this is first fragment
+                    # Raise error if not first fragment with rsv1 = 0x1
+                    if self._frame_fin or self._compressed is None:
+                        self._compressed = True if rsv1 else False
+                    elif rsv1:
+                        raise WebSocketError(
+                            WSCloseCode.PROTOCOL_ERROR,
+                            'Received frame with non-zero reserved bits')
+
+                    self._frame_fin = bool(fin)
                     self._frame_opcode = opcode
-                    self._has_mask = has_mask
+                    self._has_mask = bool(has_mask)
                     self._payload_length_flag = length
                     self._state = WSParserState.READ_PAYLOAD_LENGTH
                 else:
@@ -394,37 +522,69 @@ class WebSocketReader:
 
                 if self._payload_length == 0:
                     if self._has_mask:
-                        payload = _websocket_mask(
-                            self._frame_mask, payload)
+                        assert self._frame_mask is not None
+                        _websocket_mask(self._frame_mask, payload)
 
-                    frames.append(
-                        (self._frame_fin, self._frame_opcode, payload))
+                    frames.append((
+                        self._frame_fin,
+                        self._frame_opcode,
+                        payload,
+                        self._compressed))
 
                     self._frame_payload = bytearray()
                     self._state = WSParserState.READ_HEADER
                 else:
                     break
 
+        self._tail = buf[start_pos:]
+
         return frames
 
 
 class WebSocketWriter:
 
-    def __init__(self, stream, *,
-                 use_mask=False, limit=DEFAULT_LIMIT, random=random.Random()):
-        self.stream = stream
-        self.writer = stream.transport
+    def __init__(self, protocol: BaseProtocol, transport: asyncio.Transport, *,
+                 use_mask: bool=False, limit: int=DEFAULT_LIMIT,
+                 random: Any=random.Random(),
+                 compress: int=0, notakeover: bool=False) -> None:
+        self.protocol = protocol
+        self.transport = transport
         self.use_mask = use_mask
         self.randrange = random.randrange
+        self.compress = compress
+        self.notakeover = notakeover
         self._closing = False
         self._limit = limit
         self._output_size = 0
+        self._compressobj = None  # type: Any  # actually compressobj
 
-    def _send_frame(self, message, opcode):
+    async def _send_frame(self, message: bytes, opcode: int,
+                          compress: Optional[int]=None) -> None:
         """Send a frame over the websocket with message as its payload."""
         if self._closing:
             ws_logger.warning('websocket connection is closing.')
 
+        rsv = 0
+
+        # Only compress larger packets (disabled)
+        # Does small packet needs to be compressed?
+        # if self.compress and opcode < 8 and len(message) > 124:
+        if (compress or self.compress) and opcode < 8:
+            if compress:
+                # Do not set self._compress if compressing is for this frame
+                compressobj = zlib.compressobj(wbits=-compress)
+            else:  # self.compress
+                if not self._compressobj:
+                    self._compressobj = zlib.compressobj(wbits=-self.compress)
+                compressobj = self._compressobj
+
+            message = compressobj.compress(message)
+            message = message + compressobj.flush(
+                zlib.Z_FULL_FLUSH if self.notakeover else zlib.Z_SYNC_FLUSH)
+            if message.endswith(_WS_DEFLATE_TRAILING):
+                message = message[:-4]
+            rsv = rsv | 0x40
+
         msg_length = len(message)
 
         use_mask = self.use_mask
@@ -434,138 +594,60 @@ class WebSocketWriter:
             mask_bit = 0
 
         if msg_length < 126:
-            header = PACK_LEN1(0x80 | opcode, msg_length | mask_bit)
+            header = PACK_LEN1(0x80 | rsv | opcode, msg_length | mask_bit)
         elif msg_length < (1 << 16):
-            header = PACK_LEN2(0x80 | opcode, 126 | mask_bit, msg_length)
+            header = PACK_LEN2(0x80 | rsv | opcode, 126 | mask_bit, msg_length)
         else:
-            header = PACK_LEN3(0x80 | opcode, 127 | mask_bit, msg_length)
+            header = PACK_LEN3(0x80 | rsv | opcode, 127 | mask_bit, msg_length)
         if use_mask:
             mask = self.randrange(0, 0xffffffff)
             mask = mask.to_bytes(4, 'big')
-            message = _websocket_mask(mask, bytearray(message))
-            self.writer.write(header + mask + message)
+            message = bytearray(message)
+            _websocket_mask(mask, message)
+            self.transport.write(header + mask + message)
             self._output_size += len(header) + len(mask) + len(message)
         else:
             if len(message) > MSG_SIZE:
-                self.writer.write(header)
-                self.writer.write(message)
+                self.transport.write(header)
+                self.transport.write(message)
             else:
-                self.writer.write(header + message)
+                self.transport.write(header + message)
 
             self._output_size += len(header) + len(message)
 
         if self._output_size > self._limit:
             self._output_size = 0
-            return self.stream.drain()
+            await self.protocol._drain_helper()
 
-        return noop()
-
-    def pong(self, message=b''):
+    async def pong(self, message: bytes=b'') -> None:
         """Send pong message."""
         if isinstance(message, str):
             message = message.encode('utf-8')
-        return self._send_frame(message, WSMsgType.PONG)
+        await self._send_frame(message, WSMsgType.PONG)
 
-    def ping(self, message=b''):
+    async def ping(self, message: bytes=b'') -> None:
         """Send ping message."""
         if isinstance(message, str):
             message = message.encode('utf-8')
-        return self._send_frame(message, WSMsgType.PING)
+        await self._send_frame(message, WSMsgType.PING)
 
-    def send(self, message, binary=False):
+    async def send(self, message: Union[str, bytes],
+                   binary: bool=False,
+                   compress: Optional[int]=None) -> None:
         """Send a frame over the websocket with message as its payload."""
         if isinstance(message, str):
             message = message.encode('utf-8')
         if binary:
-            return self._send_frame(message, WSMsgType.BINARY)
+            await self._send_frame(message, WSMsgType.BINARY, compress)
         else:
-            return self._send_frame(message, WSMsgType.TEXT)
+            await self._send_frame(message, WSMsgType.TEXT, compress)
 
-    def close(self, code=1000, message=b''):
+    async def close(self, code: int=1000, message: bytes=b'') -> None:
         """Close the websocket, sending the specified code and message."""
         if isinstance(message, str):
             message = message.encode('utf-8')
         try:
-            return self._send_frame(
+            await self._send_frame(
                 PACK_CLOSE_CODE(code) + message, opcode=WSMsgType.CLOSE)
         finally:
             self._closing = True
-
-
-def do_handshake(method, headers, stream,
-                 protocols=(), write_buffer_size=DEFAULT_LIMIT):
-    """Prepare WebSocket handshake.
-
-    It return HTTP response code, response headers, websocket parser,
-    websocket writer. It does not perform any IO.
-
-    `protocols` is a sequence of known protocols. On successful handshake,
-    the returned response headers contain the first protocol in this list
-    which the server also knows.
-
-    `write_buffer_size` max size of write buffer before `drain()` get called.
-    """
-    # WebSocket accepts only GET
-    if method.upper() != hdrs.METH_GET:
-        raise HttpProcessingError(
-            code=405, headers=((hdrs.ALLOW, hdrs.METH_GET),))
-
-    if 'websocket' != headers.get(hdrs.UPGRADE, '').lower().strip():
-        raise HttpBadRequest(
-            message='No WebSocket UPGRADE hdr: {}\n Can '
-            '"Upgrade" only to "WebSocket".'.format(headers.get(hdrs.UPGRADE)))
-
-    if 'upgrade' not in headers.get(hdrs.CONNECTION, '').lower():
-        raise HttpBadRequest(
-            message='No CONNECTION upgrade hdr: {}'.format(
-                headers.get(hdrs.CONNECTION)))
-
-    # find common sub-protocol between client and server
-    protocol = None
-    if hdrs.SEC_WEBSOCKET_PROTOCOL in headers:
-        req_protocols = [str(proto.strip()) for proto in
-                         headers[hdrs.SEC_WEBSOCKET_PROTOCOL].split(',')]
-
-        for proto in req_protocols:
-            if proto in protocols:
-                protocol = proto
-                break
-        else:
-            # No overlap found: Return no protocol as per spec
-            ws_logger.warning(
-                'Client protocols %r don’t overlap server-known ones %r',
-                req_protocols, protocols)
-
-    # check supported version
-    version = headers.get(hdrs.SEC_WEBSOCKET_VERSION, '')
-    if version not in ('13', '8', '7'):
-        raise HttpBadRequest(
-            message='Unsupported version: {}'.format(version),
-            headers=((hdrs.SEC_WEBSOCKET_VERSION, '13'),))
-
-    # check client handshake for validity
-    key = headers.get(hdrs.SEC_WEBSOCKET_KEY)
-    try:
-        if not key or len(base64.b64decode(key)) != 16:
-            raise HttpBadRequest(
-                message='Handshake error: {!r}'.format(key))
-    except binascii.Error:
-        raise HttpBadRequest(
-            message='Handshake error: {!r}'.format(key)) from None
-
-    response_headers = [
-        (hdrs.UPGRADE, 'websocket'),
-        (hdrs.CONNECTION, 'upgrade'),
-        (hdrs.TRANSFER_ENCODING, 'chunked'),
-        (hdrs.SEC_WEBSOCKET_ACCEPT, base64.b64encode(
-            hashlib.sha1(key.encode() + WS_KEY).digest()).decode())]
-
-    if protocol:
-        response_headers.append((hdrs.SEC_WEBSOCKET_PROTOCOL, protocol))
-
-    # response code, headers, None, writer, protocol
-    return (101,
-            response_headers,
-            None,
-            WebSocketWriter(stream, limit=write_buffer_size),
-            protocol)
