@@ -21,7 +21,7 @@ def session():
     return mock.Mock()
 
 
-async def test_http_processing_error(session):
+async def test_http_processing_error(session) -> None:
     loop = mock.Mock()
     request_info = mock.Mock()
     response = ClientResponse(
@@ -36,7 +36,7 @@ async def test_http_processing_error(session):
     loop.get_debug.return_value = True
 
     connection = mock.Mock()
-    connection.protocol = aiohttp.DataQueue(loop=loop)
+    connection.protocol = aiohttp.DataQueue(loop)
     connection.protocol.set_response_params = mock.Mock()
     connection.protocol.set_exception(http.HttpProcessingError())
 
@@ -46,7 +46,7 @@ async def test_http_processing_error(session):
     assert info.value.request_info is request_info
 
 
-def test_del(session):
+def test_del(session) -> None:
     loop = mock.Mock()
     response = ClientResponse('get', URL('http://del-cl-resp.org'),
                               request_info=mock.Mock(),
@@ -71,7 +71,7 @@ def test_del(session):
     connection.release.assert_called_with()
 
 
-def test_close(loop, session):
+def test_close(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -88,7 +88,7 @@ def test_close(loop, session):
     response.close()
 
 
-def test_wait_for_100_1(loop, session):
+def test_wait_for_100_1(loop, session) -> None:
     response = ClientResponse(
         'get', URL('http://python.org'), continue100=object(),
         request_info=mock.Mock(),
@@ -101,7 +101,7 @@ def test_wait_for_100_1(loop, session):
     response.close()
 
 
-def test_wait_for_100_2(loop, session):
+def test_wait_for_100_2(loop, session) -> None:
     response = ClientResponse(
         'get', URL('http://python.org'),
         request_info=mock.Mock(),
@@ -115,7 +115,7 @@ def test_wait_for_100_2(loop, session):
     response.close()
 
 
-def test_repr(loop, session):
+def test_repr(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -130,7 +130,7 @@ def test_repr(loop, session):
         in repr(response)
 
 
-def test_repr_non_ascii_url():
+def test_repr_non_ascii_url() -> None:
     response = ClientResponse('get', URL('http://fake-host.org/\u03bb'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -143,7 +143,7 @@ def test_repr_non_ascii_url():
         in repr(response)
 
 
-def test_repr_non_ascii_reason():
+def test_repr_non_ascii_reason() -> None:
     response = ClientResponse('get', URL('http://fake-host.org/path'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -157,7 +157,7 @@ def test_repr_non_ascii_reason():
         in repr(response)
 
 
-def test_url_obj_deprecated():
+def test_url_obj_deprecated() -> None:
     response = ClientResponse('get', URL('http://fake-host.org/'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -170,7 +170,7 @@ def test_url_obj_deprecated():
         response.url_obj
 
 
-async def test_read_and_release_connection(loop, session):
+async def test_read_and_release_connection(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -192,7 +192,7 @@ async def test_read_and_release_connection(loop, session):
     assert response._connection is None
 
 
-async def test_read_and_release_connection_with_error(loop, session):
+async def test_read_and_release_connection_with_error(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -210,7 +210,7 @@ async def test_read_and_release_connection_with_error(loop, session):
     assert response._closed
 
 
-async def test_release(loop, session):
+async def test_release(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -230,7 +230,7 @@ async def test_release(loop, session):
 
 @pytest.mark.skipif(sys.implementation.name != 'cpython',
                     reason="Other implementations has different GC strategies")
-async def test_release_on_del(loop, session):
+async def test_release_on_del(loop, session) -> None:
     connection = mock.Mock()
     connection.protocol.upgraded = False
 
@@ -251,7 +251,7 @@ async def test_release_on_del(loop, session):
     assert connection.release.called
 
 
-async def test_response_eof(loop, session):
+async def test_response_eof(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -269,7 +269,7 @@ async def test_response_eof(loop, session):
     assert response._connection is None
 
 
-async def test_response_eof_upgraded(loop, session):
+async def test_response_eof_upgraded(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -287,7 +287,7 @@ async def test_response_eof_upgraded(loop, session):
     assert response._connection is conn
 
 
-async def test_response_eof_after_connection_detach(loop, session):
+async def test_response_eof_after_connection_detach(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -305,7 +305,7 @@ async def test_response_eof_after_connection_detach(loop, session):
     assert response._connection is None
 
 
-async def test_text(loop, session):
+async def test_text(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -330,7 +330,7 @@ async def test_text(loop, session):
     assert response._connection is None
 
 
-async def test_text_bad_encoding(loop, session):
+async def test_text_bad_encoding(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -358,7 +358,7 @@ async def test_text_bad_encoding(loop, session):
     assert response._connection is None
 
 
-async def test_text_custom_encoding(loop, session):
+async def test_text_custom_encoding(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -385,7 +385,7 @@ async def test_text_custom_encoding(loop, session):
     assert not response.get_encoding.called
 
 
-async def test_text_detect_encoding(loop, session):
+async def test_text_detect_encoding(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -410,7 +410,7 @@ async def test_text_detect_encoding(loop, session):
     assert response._connection is None
 
 
-async def test_text_detect_encoding_if_invalid_charset(loop, session):
+async def test_text_detect_encoding_if_invalid_charset(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -436,7 +436,7 @@ async def test_text_detect_encoding_if_invalid_charset(loop, session):
     assert response.get_encoding().lower() in ('windows-1251', 'maccyrillic')
 
 
-async def test_text_after_read(loop, session):
+async def test_text_after_read(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -461,7 +461,7 @@ async def test_text_after_read(loop, session):
     assert response._connection is None
 
 
-async def test_json(loop, session):
+async def test_json(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -486,7 +486,7 @@ async def test_json(loop, session):
     assert response._connection is None
 
 
-async def test_json_extended_content_type(loop, session):
+async def test_json_extended_content_type(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -512,7 +512,7 @@ async def test_json_extended_content_type(loop, session):
     assert response._connection is None
 
 
-async def test_json_custom_content_type(loop, session):
+async def test_json_custom_content_type(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -537,7 +537,7 @@ async def test_json_custom_content_type(loop, session):
     assert response._connection is None
 
 
-async def test_json_custom_loader(loop, session):
+async def test_json_custom_loader(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -557,7 +557,7 @@ async def test_json_custom_loader(loop, session):
     assert res == 'data-custom'
 
 
-async def test_json_invalid_content_type(loop, session):
+async def test_json_invalid_content_type(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -576,7 +576,7 @@ async def test_json_invalid_content_type(loop, session):
     assert info.value.request_info == response.request_info
 
 
-async def test_json_no_content(loop, session):
+async def test_json_no_content(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -593,7 +593,7 @@ async def test_json_no_content(loop, session):
     assert res is None
 
 
-async def test_json_override_encoding(loop, session):
+async def test_json_override_encoding(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -620,7 +620,7 @@ async def test_json_override_encoding(loop, session):
     assert not response.get_encoding.called
 
 
-def test_get_encoding_unknown(loop, session):
+def test_get_encoding_unknown(loop, session) -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -636,7 +636,7 @@ def test_get_encoding_unknown(loop, session):
         assert response.get_encoding() == 'utf-8'
 
 
-def test_raise_for_status_2xx():
+def test_raise_for_status_2xx() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -650,7 +650,7 @@ def test_raise_for_status_2xx():
     response.raise_for_status()  # should not raise
 
 
-def test_raise_for_status_4xx():
+def test_raise_for_status_4xx() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -665,9 +665,28 @@ def test_raise_for_status_4xx():
         response.raise_for_status()
     assert str(cm.value.status) == '409'
     assert str(cm.value.message) == "CONFLICT"
+    assert response.closed
 
 
-def test_resp_host():
+def test_raise_for_status_4xx_without_reason() -> None:
+    response = ClientResponse('get', URL('http://def-cl-resp.org'),
+                              request_info=mock.Mock(),
+                              writer=mock.Mock(),
+                              continue100=None,
+                              timer=TimerNoop(),
+                              traces=[],
+                              loop=mock.Mock(),
+                              session=mock.Mock())
+    response.status = 404
+    response.reason = ''
+    with pytest.raises(aiohttp.ClientResponseError) as cm:
+        response.raise_for_status()
+    assert str(cm.value.status) == '404'
+    assert str(cm.value.message) == ''
+    assert response.closed
+
+
+def test_resp_host() -> None:
     response = ClientResponse('get', URL('http://del-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -679,7 +698,7 @@ def test_resp_host():
     assert 'del-cl-resp.org' == response.host
 
 
-def test_content_type():
+def test_content_type() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -693,7 +712,7 @@ def test_content_type():
     assert 'application/json' == response.content_type
 
 
-def test_content_type_no_header():
+def test_content_type_no_header() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -707,7 +726,7 @@ def test_content_type_no_header():
     assert 'application/octet-stream' == response.content_type
 
 
-def test_charset():
+def test_charset() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -721,7 +740,7 @@ def test_charset():
     assert 'cp1251' == response.charset
 
 
-def test_charset_no_header():
+def test_charset_no_header() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -735,7 +754,7 @@ def test_charset_no_header():
     assert response.charset is None
 
 
-def test_charset_no_charset():
+def test_charset_no_charset() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -749,7 +768,7 @@ def test_charset_no_charset():
     assert response.charset is None
 
 
-def test_content_disposition_full():
+def test_content_disposition_full() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -768,7 +787,7 @@ def test_content_disposition_full():
         response.content_disposition.parameters["foo"] = "baz"
 
 
-def test_content_disposition_no_parameters():
+def test_content_disposition_no_parameters() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -784,7 +803,7 @@ def test_content_disposition_no_parameters():
     assert {} == response.content_disposition.parameters
 
 
-def test_content_disposition_no_header():
+def test_content_disposition_no_header() -> None:
     response = ClientResponse('get', URL('http://def-cl-resp.org'),
                               request_info=mock.Mock(),
                               writer=mock.Mock(),
@@ -798,7 +817,7 @@ def test_content_disposition_no_header():
     assert response.content_disposition is None
 
 
-def test_response_request_info():
+def test_response_request_info() -> None:
     url = 'http://def-cl-resp.org'
     headers = {'Content-Type': 'application/json;charset=cp1251'}
     response = ClientResponse(
@@ -820,7 +839,7 @@ def test_response_request_info():
     assert headers == response.request_info.headers
 
 
-def test_request_info_in_exception():
+def test_request_info_in_exception() -> None:
     url = 'http://def-cl-resp.org'
     headers = {'Content-Type': 'application/json;charset=cp1251'}
     response = ClientResponse(
@@ -845,7 +864,7 @@ def test_request_info_in_exception():
     assert cm.value.request_info == response.request_info
 
 
-def test_no_redirect_history_in_exception():
+def test_no_redirect_history_in_exception() -> None:
     url = 'http://def-cl-resp.org'
     headers = {'Content-Type': 'application/json;charset=cp1251'}
     response = ClientResponse(
@@ -870,7 +889,7 @@ def test_no_redirect_history_in_exception():
     assert () == cm.value.history
 
 
-def test_redirect_history_in_exception():
+def test_redirect_history_in_exception() -> None:
     hist_url = 'http://def-cl-resp.org'
     url = 'http://def-cl-resp.org/index.htm'
     hist_headers = {'Content-Type': 'application/json;charset=cp1251',
@@ -921,7 +940,7 @@ def test_redirect_history_in_exception():
     assert [hist_response] == cm.value.history
 
 
-async def test_response_read_triggers_callback(loop, session):
+async def test_response_read_triggers_callback(loop, session) -> None:
     trace = mock.Mock()
     trace.send_response_chunk_received = make_mocked_coro()
     response_body = b'This is response'
@@ -958,7 +977,7 @@ async def test_response_read_triggers_callback(loop, session):
     )
 
 
-def test_response_real_url(loop, session):
+def test_response_real_url(loop, session) -> None:
     url = URL('http://def-cl-resp.org/#urlfragment')
     response = ClientResponse('get', url,
                               request_info=mock.Mock(),
@@ -972,7 +991,7 @@ def test_response_real_url(loop, session):
     assert response.real_url == url
 
 
-def test_response_links_comma_separated(loop, session):
+def test_response_links_comma_separated(loop, session) -> None:
     url = URL('http://def-cl-resp.org/')
     response = ClientResponse('get', url,
                               request_info=mock.Mock(),
@@ -1001,7 +1020,7 @@ def test_response_links_comma_separated(loop, session):
     )
 
 
-def test_response_links_multiple_headers(loop, session):
+def test_response_links_multiple_headers(loop, session) -> None:
     url = URL('http://def-cl-resp.org/')
     response = ClientResponse('get', url,
                               request_info=mock.Mock(),
@@ -1033,7 +1052,7 @@ def test_response_links_multiple_headers(loop, session):
     )
 
 
-def test_response_links_no_rel(loop, session):
+def test_response_links_no_rel(loop, session) -> None:
     url = URL('http://def-cl-resp.org/')
     response = ClientResponse('get', url,
                               request_info=mock.Mock(),
@@ -1058,7 +1077,7 @@ def test_response_links_no_rel(loop, session):
     )
 
 
-def test_response_links_quoted(loop, session):
+def test_response_links_quoted(loop, session) -> None:
     url = URL('http://def-cl-resp.org/')
     response = ClientResponse('get', url,
                               request_info=mock.Mock(),
@@ -1083,7 +1102,7 @@ def test_response_links_quoted(loop, session):
     )
 
 
-def test_response_links_relative(loop, session):
+def test_response_links_relative(loop, session) -> None:
     url = URL('http://def-cl-resp.org/')
     response = ClientResponse('get', url,
                               request_info=mock.Mock(),
@@ -1108,7 +1127,7 @@ def test_response_links_relative(loop, session):
     )
 
 
-def test_response_links_empty(loop, session):
+def test_response_links_empty(loop, session) -> None:
     url = URL('http://def-cl-resp.org/')
     response = ClientResponse('get', url,
                               request_info=mock.Mock(),
