@@ -15,7 +15,7 @@ def ceil(mocker):
     mocker.patch('aiohttp.helpers.ceil').side_effect = ceil
 
 
-async def test_send_recv_text(loop, aiohttp_client):
+async def test_send_recv_text(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -41,7 +41,7 @@ async def test_send_recv_text(loop, aiohttp_client):
     assert resp.get_extra_info('socket') is None
 
 
-async def test_send_recv_bytes_bad_type(loop, aiohttp_client):
+async def test_send_recv_bytes_bad_type(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -63,7 +63,7 @@ async def test_send_recv_bytes_bad_type(loop, aiohttp_client):
         await resp.close()
 
 
-async def test_send_recv_bytes(loop, aiohttp_client):
+async def test_send_recv_bytes(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -87,7 +87,7 @@ async def test_send_recv_bytes(loop, aiohttp_client):
     await resp.close()
 
 
-async def test_send_recv_text_bad_type(loop, aiohttp_client):
+async def test_send_recv_text_bad_type(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -111,7 +111,7 @@ async def test_send_recv_text_bad_type(loop, aiohttp_client):
         await resp.close()
 
 
-async def test_send_recv_json(loop, aiohttp_client):
+async def test_send_recv_json(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -134,8 +134,8 @@ async def test_send_recv_json(loop, aiohttp_client):
     await resp.close()
 
 
-async def test_ping_pong(loop, aiohttp_client):
-
+async def test_ping_pong(aiohttp_client) -> None:
+    loop = asyncio.get_event_loop()
     closed = loop.create_future()
 
     async def handler(request):
@@ -170,8 +170,8 @@ async def test_ping_pong(loop, aiohttp_client):
     await closed
 
 
-async def test_ping_pong_manual(loop, aiohttp_client):
-
+async def test_ping_pong_manual(aiohttp_client) -> None:
+    loop = asyncio.get_event_loop()
     closed = loop.create_future()
 
     async def handler(request):
@@ -211,7 +211,7 @@ async def test_ping_pong_manual(loop, aiohttp_client):
     await closed
 
 
-async def test_close(loop, aiohttp_client):
+async def test_close(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -239,7 +239,7 @@ async def test_close(loop, aiohttp_client):
     assert msg.type == aiohttp.WSMsgType.CLOSED
 
 
-async def test_concurrent_close(loop, aiohttp_client):
+async def test_concurrent_close(aiohttp_client) -> None:
     client_ws = None
 
     async def handler(request):
@@ -266,13 +266,13 @@ async def test_concurrent_close(loop, aiohttp_client):
     msg = await ws.receive()
     assert msg.type == aiohttp.WSMsgType.CLOSING
 
-    await asyncio.sleep(0.01, loop=loop)
+    await asyncio.sleep(0.01)
     msg = await ws.receive()
     assert msg.type == aiohttp.WSMsgType.CLOSED
 
 
-async def test_close_from_server(loop, aiohttp_client):
-
+async def test_close_from_server(aiohttp_client) -> None:
+    loop = asyncio.get_event_loop()
     closed = loop.create_future()
 
     async def handler(request):
@@ -303,8 +303,8 @@ async def test_close_from_server(loop, aiohttp_client):
     await closed
 
 
-async def test_close_manual(loop, aiohttp_client):
-
+async def test_close_manual(aiohttp_client) -> None:
+    loop = asyncio.get_event_loop()
     closed = loop.create_future()
 
     async def handler(request):
@@ -340,14 +340,14 @@ async def test_close_manual(loop, aiohttp_client):
     assert resp.closed
 
 
-async def test_close_timeout(loop, aiohttp_client):
+async def test_close_timeout(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
         await ws.prepare(request)
         await ws.receive_bytes()
         await ws.send_str('test')
-        await asyncio.sleep(1, loop=loop)
+        await asyncio.sleep(1)
         return ws
 
     app = web.Application()
@@ -366,14 +366,15 @@ async def test_close_timeout(loop, aiohttp_client):
     assert isinstance(resp.exception(), asyncio.TimeoutError)
 
 
-async def test_close_cancel(loop, aiohttp_client):
+async def test_close_cancel(aiohttp_client) -> None:
+    loop = asyncio.get_event_loop()
 
     async def handler(request):
         ws = web.WebSocketResponse()
         await ws.prepare(request)
         await ws.receive_bytes()
         await ws.send_str('test')
-        await asyncio.sleep(10, loop=loop)
+        await asyncio.sleep(10)
 
     app = web.Application()
     app.router.add_route('GET', '/', handler)
@@ -386,14 +387,14 @@ async def test_close_cancel(loop, aiohttp_client):
     assert text.data == 'test'
 
     t = loop.create_task(resp.close())
-    await asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1)
     t.cancel()
-    await asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1)
     assert resp.closed
     assert resp.exception() is None
 
 
-async def test_override_default_headers(loop, aiohttp_client):
+async def test_override_default_headers(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.headers[hdrs.SEC_WEBSOCKET_VERSION] == '8'
@@ -413,7 +414,7 @@ async def test_override_default_headers(loop, aiohttp_client):
     await resp.close()
 
 
-async def test_additional_headers(loop, aiohttp_client):
+async def test_additional_headers(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.headers['x-hdr'] == 'xtra'
@@ -433,7 +434,7 @@ async def test_additional_headers(loop, aiohttp_client):
     await resp.close()
 
 
-async def test_recv_protocol_error(loop, aiohttp_client):
+async def test_recv_protocol_error(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -458,7 +459,7 @@ async def test_recv_protocol_error(loop, aiohttp_client):
     await resp.close()
 
 
-async def test_recv_timeout(loop, aiohttp_client):
+async def test_recv_timeout(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -466,7 +467,7 @@ async def test_recv_timeout(loop, aiohttp_client):
 
         await ws.receive_str()
 
-        await asyncio.sleep(0.1, loop=request.app.loop)
+        await asyncio.sleep(0.1)
 
         await ws.close()
         return ws
@@ -478,13 +479,13 @@ async def test_recv_timeout(loop, aiohttp_client):
     await resp.send_str('ask')
 
     with pytest.raises(asyncio.TimeoutError):
-        with async_timeout.timeout(0.01, loop=app.loop):
+        with async_timeout.timeout(0.01):
             await resp.receive()
 
     await resp.close()
 
 
-async def test_receive_timeout(loop, aiohttp_client):
+async def test_receive_timeout(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -505,7 +506,7 @@ async def test_receive_timeout(loop, aiohttp_client):
     await resp.close()
 
 
-async def test_custom_receive_timeout(loop, aiohttp_client):
+async def test_custom_receive_timeout(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -526,7 +527,7 @@ async def test_custom_receive_timeout(loop, aiohttp_client):
     await resp.close()
 
 
-async def test_heartbeat(loop, aiohttp_client, ceil):
+async def test_heartbeat(aiohttp_client, ceil) -> None:
     ping_received = False
 
     async def handler(request):
@@ -551,7 +552,7 @@ async def test_heartbeat(loop, aiohttp_client, ceil):
     assert ping_received
 
 
-async def test_heartbeat_no_pong(loop, aiohttp_client, ceil):
+async def test_heartbeat_no_pong(aiohttp_client, ceil) -> None:
     ping_received = False
 
     async def handler(request):
@@ -576,7 +577,7 @@ async def test_heartbeat_no_pong(loop, aiohttp_client, ceil):
     assert ping_received
 
 
-async def test_send_recv_compress(loop, aiohttp_client):
+async def test_send_recv_compress(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -602,7 +603,7 @@ async def test_send_recv_compress(loop, aiohttp_client):
     assert resp.get_extra_info('socket') is None
 
 
-async def test_send_recv_compress_wbits(loop, aiohttp_client):
+async def test_send_recv_compress_wbits(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -630,7 +631,7 @@ async def test_send_recv_compress_wbits(loop, aiohttp_client):
     assert resp.get_extra_info('socket') is None
 
 
-async def test_send_recv_compress_wbit_error(loop, aiohttp_client):
+async def test_send_recv_compress_wbit_error(aiohttp_client) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -648,7 +649,7 @@ async def test_send_recv_compress_wbit_error(loop, aiohttp_client):
         await client.ws_connect('/', compress=1)
 
 
-async def test_ws_client_async_for(loop, aiohttp_client):
+async def test_ws_client_async_for(aiohttp_client) -> None:
     items = ['q1', 'q2', 'q3']
 
     async def handler(request):
@@ -674,7 +675,7 @@ async def test_ws_client_async_for(loop, aiohttp_client):
     assert resp.closed
 
 
-async def test_ws_async_with(loop, aiohttp_server):
+async def test_ws_async_with(aiohttp_server) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -689,7 +690,7 @@ async def test_ws_async_with(loop, aiohttp_server):
 
     server = await aiohttp_server(app)
 
-    async with aiohttp.ClientSession(loop=loop) as client:
+    async with aiohttp.ClientSession() as client:
         async with client.ws_connect(server.make_url('/')) as ws:
             await ws.send_str('request')
             msg = await ws.receive()
@@ -698,7 +699,7 @@ async def test_ws_async_with(loop, aiohttp_server):
         assert ws.closed
 
 
-async def test_ws_async_with_send(loop, aiohttp_server):
+async def test_ws_async_with_send(aiohttp_server) -> None:
     # send_xxx methods have to return awaitable objects
 
     async def handler(request):
@@ -714,7 +715,7 @@ async def test_ws_async_with_send(loop, aiohttp_server):
 
     server = await aiohttp_server(app)
 
-    async with aiohttp.ClientSession(loop=loop) as client:
+    async with aiohttp.ClientSession() as client:
         async with client.ws_connect(server.make_url('/')) as ws:
             await ws.send_str('request')
             msg = await ws.receive()
@@ -723,7 +724,7 @@ async def test_ws_async_with_send(loop, aiohttp_server):
         assert ws.closed
 
 
-async def test_ws_async_with_shortcut(loop, aiohttp_server):
+async def test_ws_async_with_shortcut(aiohttp_server) -> None:
 
     async def handler(request):
         ws = web.WebSocketResponse()
@@ -737,7 +738,7 @@ async def test_ws_async_with_shortcut(loop, aiohttp_server):
     app.router.add_route('GET', '/', handler)
     server = await aiohttp_server(app)
 
-    async with aiohttp.ClientSession(loop=loop) as client:
+    async with aiohttp.ClientSession() as client:
         async with client.ws_connect(server.make_url('/')) as ws:
             await ws.send_str('request')
             msg = await ws.receive()
@@ -746,8 +747,8 @@ async def test_ws_async_with_shortcut(loop, aiohttp_server):
         assert ws.closed
 
 
-async def test_closed_async_for(loop, aiohttp_client):
-
+async def test_closed_async_for(aiohttp_client) -> None:
+    loop = asyncio.get_event_loop()
     closed = loop.create_future()
 
     async def handler(request):
@@ -779,3 +780,52 @@ async def test_closed_async_for(loop, aiohttp_client):
     assert resp.closed
 
     await closed
+
+
+async def test_peer_connection_lost(aiohttp_client) -> None:
+
+    async def handler(request):
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+
+        msg = await ws.receive_str()
+        assert msg == 'ask'
+        await ws.send_str('answer')
+        request.transport.close()
+        await asyncio.sleep(10)
+        return ws
+
+    app = web.Application()
+    app.router.add_route('GET', '/', handler)
+    client = await aiohttp_client(app)
+    resp = await client.ws_connect('/')
+    await resp.send_str('ask')
+    assert 'answer' == await resp.receive_str()
+
+    msg = await resp.receive()
+    assert msg.type == aiohttp.WSMsgType.CLOSED
+    await resp.close()
+
+
+async def test_peer_connection_lost_iter(aiohttp_client) -> None:
+
+    async def handler(request):
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+
+        msg = await ws.receive_str()
+        assert msg == 'ask'
+        await ws.send_str('answer')
+        request.transport.close()
+        await asyncio.sleep(100)
+        return ws
+
+    app = web.Application()
+    app.router.add_route('GET', '/', handler)
+    client = await aiohttp_client(app)
+    resp = await client.ws_connect('/')
+    await resp.send_str('ask')
+    async for msg in resp:
+        assert 'answer' == msg.data
+
+    await resp.close()
