@@ -4,13 +4,13 @@ import itertools
 import os
 import tempfile
 import unittest
+from http.cookies import SimpleCookie
 from unittest import mock
 
 import pytest
 from yarl import URL
 
 from aiohttp import CookieJar, DummyCookieJar
-from aiohttp.helpers import SimpleCookie
 
 
 @pytest.fixture
@@ -54,7 +54,7 @@ def cookies_to_receive():
     )
 
 
-def test_date_parsing():
+def test_date_parsing() -> None:
     parse_func = CookieJar._parse_date
     utc = datetime.timezone.utc
 
@@ -98,7 +98,7 @@ def test_date_parsing():
     assert parse_func("Tue, 1 Jan 1970 77:88:99 GMT") is None
 
 
-def test_domain_matching():
+def test_domain_matching() -> None:
     test_func = CookieJar._is_domain_match
 
     assert test_func("test.com", "test.com")
@@ -111,7 +111,7 @@ def test_domain_matching():
     assert not test_func("test.com", "127.0.0.1")
 
 
-def test_path_matching():
+def test_path_matching() -> None:
     test_func = CookieJar._is_path_match
 
     assert test_func("/", "")
@@ -133,7 +133,7 @@ def test_path_matching():
     assert not test_func("/different-folder/", "/folder/")
 
 
-def test_constructor(loop, cookies_to_send, cookies_to_receive):
+async def test_constructor(loop, cookies_to_send, cookies_to_receive) -> None:
     jar = CookieJar(loop=loop)
     jar.update_cookies(cookies_to_send)
     jar_cookies = SimpleCookie()
@@ -144,7 +144,7 @@ def test_constructor(loop, cookies_to_send, cookies_to_receive):
     assert jar._loop is loop
 
 
-def test_save_load(loop, cookies_to_send, cookies_to_receive):
+async def test_save_load(loop, cookies_to_send, cookies_to_receive) -> None:
     file_path = tempfile.mkdtemp() + '/aiohttp.test.cookie'
 
     # export cookie jar
@@ -163,7 +163,7 @@ def test_save_load(loop, cookies_to_send, cookies_to_receive):
     assert jar_test == cookies_to_receive
 
 
-def test_update_cookie_with_unicode_domain(loop):
+async def test_update_cookie_with_unicode_domain(loop) -> None:
     cookies = (
         "idna-domain-first=first; Domain=xn--9caa.com; Path=/;",
         "idna-domain-second=second; Domain=xn--9caa.com; Path=/;",
@@ -180,8 +180,8 @@ def test_update_cookie_with_unicode_domain(loop):
     assert jar_test == SimpleCookie(" ".join(cookies))
 
 
-def test_filter_cookie_with_unicode_domain(loop):
-    jar = CookieJar(loop=loop)
+async def test_filter_cookie_with_unicode_domain(loop) -> None:
+    jar = CookieJar()
     jar.update_cookies(SimpleCookie(
         "idna-domain-first=first; Domain=xn--9caa.com; Path=/; "
     ))
@@ -189,13 +189,7 @@ def test_filter_cookie_with_unicode_domain(loop):
     assert len(jar.filter_cookies(URL("http://xn--9caa.com"))) == 1
 
 
-def test_ctor_ith_default_loop(loop):
-    asyncio.set_event_loop(loop)
-    jar = CookieJar()
-    assert jar._loop is loop
-
-
-def test_domain_filter_ip_cookie_send(loop):
+async def test_domain_filter_ip_cookie_send(loop) -> None:
     jar = CookieJar(loop=loop)
     cookies = SimpleCookie(
         "shared-cookie=first; "
@@ -226,14 +220,14 @@ def test_domain_filter_ip_cookie_send(loop):
     assert cookies_sent == 'Cookie: shared-cookie=first'
 
 
-def test_domain_filter_ip_cookie_receive(loop, cookies_to_receive):
-    jar = CookieJar(loop=loop)
+async def test_domain_filter_ip_cookie_receive(cookies_to_receive) -> None:
+    jar = CookieJar()
 
     jar.update_cookies(cookies_to_receive, URL("http://1.2.3.4/"))
     assert len(jar) == 0
 
 
-def test_preserving_ip_domain_cookies(loop):
+async def test_preserving_ip_domain_cookies(loop) -> None:
     jar = CookieJar(loop=loop, unsafe=True)
     jar.update_cookies(SimpleCookie(
         "shared-cookie=first; "
@@ -245,7 +239,7 @@ def test_preserving_ip_domain_cookies(loop):
                             'Cookie: shared-cookie=first')
 
 
-def test_preserving_quoted_cookies(loop):
+async def test_preserving_quoted_cookies(loop) -> None:
     jar = CookieJar(loop=loop, unsafe=True)
     jar.update_cookies(SimpleCookie(
         "ip-cookie=\"second\"; Domain=127.0.0.1;"
@@ -255,7 +249,7 @@ def test_preserving_quoted_cookies(loop):
     assert cookies_sent == 'Cookie: ip-cookie=\"second\"'
 
 
-def test_ignore_domain_ending_with_dot(loop):
+async def test_ignore_domain_ending_with_dot(loop) -> None:
     jar = CookieJar(loop=loop, unsafe=True)
     jar.update_cookies(SimpleCookie("cookie=val; Domain=example.com.;"),
                        URL("http://www.example.com"))
@@ -272,7 +266,9 @@ class TestCookieJarBase(unittest.TestCase):
         asyncio.set_event_loop(None)
 
         # N.B. those need to be overridden in child test cases
-        self.jar = CookieJar(loop=self.loop)
+        async def make_jar():
+            return CookieJar()
+        self.jar = self.loop.run_until_complete(make_jar())
 
     def tearDown(self):
         self.loop.close()
@@ -333,7 +329,9 @@ class TestCookieJarSafe(TestCookieJarBase):
             "wrong-path-cookie=nineth; Domain=pathtest.com; Path=somepath;"
         )
 
-        self.jar = CookieJar(loop=self.loop)
+        async def make_jar():
+            return CookieJar()
+        self.jar = self.loop.run_until_complete(make_jar())
 
     def timed_request(self, url, update_time, send_time):
         with mock.patch.object(self.loop, 'time', return_value=update_time):
@@ -346,7 +344,7 @@ class TestCookieJarSafe(TestCookieJarBase):
 
         return cookies_sent
 
-    def test_domain_filter_same_host(self):
+    def test_domain_filter_same_host(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://example.com/"))
 
@@ -362,7 +360,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "dotted-domain-cookie"
         })
 
-    def test_domain_filter_same_host_and_subdomain(self):
+    def test_domain_filter_same_host_and_subdomain(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://test1.example.com/"))
 
@@ -380,7 +378,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "dotted-domain-cookie"
         })
 
-    def test_domain_filter_same_host_diff_subdomain(self):
+    def test_domain_filter_same_host_diff_subdomain(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://different.example.com/"))
 
@@ -396,7 +394,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "dotted-domain-cookie"
         })
 
-    def test_domain_filter_diff_host(self):
+    def test_domain_filter_diff_host(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://different.org/"))
 
@@ -410,7 +408,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "different-domain-cookie"
         })
 
-    def test_domain_filter_host_only(self):
+    def test_domain_filter_host_only(self) -> None:
         self.jar.update_cookies(self.cookies_to_receive,
                                 URL("http://example.com/"))
 
@@ -420,7 +418,7 @@ class TestCookieJarSafe(TestCookieJarBase):
         cookies_sent = self.jar.filter_cookies(URL("http://different.org/"))
         self.assertNotIn("unconstrained-cookie", set(cookies_sent.keys()))
 
-    def test_secure_filter(self):
+    def test_secure_filter(self) -> None:
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://secure.com/"))
 
@@ -436,7 +434,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "secure-cookie"
         })
 
-    def test_path_filter_root(self):
+    def test_path_filter_root(self) -> None:
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://pathtest.com/"))
 
@@ -446,7 +444,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path1-cookie"
         })
 
-    def test_path_filter_folder(self):
+    def test_path_filter_folder(self) -> None:
 
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://pathtest.com/one/"))
@@ -458,7 +456,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path2-cookie"
         })
 
-    def test_path_filter_file(self):
+    def test_path_filter_file(self) -> None:
 
         cookies_sent, _ = self.request_reply_with_same_url(
             "http://pathtest.com/one/two")
@@ -471,7 +469,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path3-cookie"
         })
 
-    def test_path_filter_subfolder(self):
+    def test_path_filter_subfolder(self) -> None:
 
         cookies_sent, _ = self.request_reply_with_same_url(
             "http://pathtest.com/one/two/")
@@ -485,7 +483,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path4-cookie"
         })
 
-    def test_path_filter_subsubfolder(self):
+    def test_path_filter_subsubfolder(self) -> None:
 
         cookies_sent, _ = self.request_reply_with_same_url(
             "http://pathtest.com/one/two/three/")
@@ -499,7 +497,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path4-cookie"
         })
 
-    def test_path_filter_different_folder(self):
+    def test_path_filter_different_folder(self) -> None:
 
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://pathtest.com/hundred/"))
@@ -510,7 +508,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path1-cookie"
         })
 
-    def test_path_value(self):
+    def test_path_value(self) -> None:
         _, cookies_received = (
             self.request_reply_with_same_url("http://pathtest.com/"))
 
@@ -525,7 +523,7 @@ class TestCookieJarSafe(TestCookieJarBase):
         self.assertEqual(cookies_received["path-cookie"]["path"], "/somepath")
         self.assertEqual(cookies_received["wrong-path-cookie"]["path"], "/")
 
-    def test_expires(self):
+    def test_expires(self) -> None:
         ts_before = datetime.datetime(
             1975, 1, 1, tzinfo=datetime.timezone.utc).timestamp()
 
@@ -547,7 +545,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "shared-cookie"
         })
 
-    def test_max_age(self):
+    def test_max_age(self) -> None:
         cookies_sent = self.timed_request(
             "http://maxagetest.com/", 1000, 1000)
 
@@ -563,7 +561,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "shared-cookie"
         })
 
-    def test_invalid_values(self):
+    def test_invalid_values(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://invalid-values.com/"))
 
@@ -579,7 +577,7 @@ class TestCookieJarSafe(TestCookieJarBase):
         cookie = cookies_sent["invalid-expires-cookie"]
         self.assertEqual(cookie["expires"], "")
 
-    def test_cookie_not_expired_when_added_after_removal(self):
+    def test_cookie_not_expired_when_added_after_removal(self) -> None:
         """Test case for https://github.com/aio-libs/aiohttp/issues/2084"""
         timestamps = [533588.993, 533588.993, 533588.993,
                       533588.993, 533589.093, 533589.093]
@@ -588,7 +586,9 @@ class TestCookieJarSafe(TestCookieJarBase):
         loop.time.side_effect = itertools.chain(
             timestamps, itertools.cycle([timestamps[-1]]))
 
-        jar = CookieJar(unsafe=True, loop=loop)
+        async def make_jar():
+            return CookieJar(unsafe=True)
+        jar = self.loop.run_until_complete(make_jar())
         # Remove `foo` cookie.
         jar.update_cookies(SimpleCookie('foo=""; Max-Age=0'))
         # Set `foo` cookie to `bar`.
@@ -598,13 +598,13 @@ class TestCookieJarSafe(TestCookieJarBase):
         assert len(jar) == 1
 
 
-def test_dummy_cookie_jar(loop):
+async def test_dummy_cookie_jar() -> None:
     cookie = SimpleCookie('foo=bar; Domain=example.com;')
-    dummy_jar = DummyCookieJar(loop=loop)
+    dummy_jar = DummyCookieJar()
     assert len(dummy_jar) == 0
     dummy_jar.update_cookies(cookie)
     assert len(dummy_jar) == 0
     with pytest.raises(StopIteration):
         next(iter(dummy_jar))
-    assert dummy_jar.filter_cookies(URL("http://example.com/")) is None
+    assert not dummy_jar.filter_cookies(URL("http://example.com/"))
     dummy_jar.clear()
