@@ -3,17 +3,17 @@
 from aiohttp import http_exceptions
 
 
-def test_bad_status_line1():
+def test_bad_status_line1() -> None:
     err = http_exceptions.BadStatusLine(b'')
     assert str(err) == "b''"
 
 
-def test_bad_status_line2():
+def test_bad_status_line2() -> None:
     err = http_exceptions.BadStatusLine('Test')
     assert str(err) == 'Test'
 
 
-def test_http_error_exception():
+def test_http_error_exception() -> None:
     exc = http_exceptions.HttpProcessingError(
         code=500, message='Internal error')
     assert exc.code == 500
