@@ -32,13 +32,14 @@ def protocol(loop, transport):
     return protocol
 
 
-def test_payloadwriter_properties(transport, protocol, loop):
+def test_payloadwriter_properties(transport,
+                                  protocol, loop) -> None:
     writer = http.StreamWriter(protocol, loop)
     assert writer.protocol == protocol
     assert writer.transport == transport
 
 
-async def test_write_payload_eof(transport, protocol, loop):
+async def test_write_payload_eof(transport, protocol, loop) -> None:
     write = transport.write = mock.Mock()
     msg = http.StreamWriter(protocol, loop)
 
@@ -50,7 +51,7 @@ async def test_write_payload_eof(transport, protocol, loop):
     assert b'data1data2' == content.split(b'\r\n\r\n', 1)[-1]
 
 
-async def test_write_payload_chunked(buf, protocol, transport, loop):
+async def test_write_payload_chunked(buf, protocol, transport, loop) -> None:
     msg = http.StreamWriter(protocol, loop)
     msg.enable_chunking()
     await msg.write(b'data')
@@ -59,7 +60,9 @@ async def test_write_payload_chunked(buf, protocol, transport, loop):
     assert b'4\r\ndata\r\n0\r\n\r\n' == buf
 
 
-async def test_write_payload_chunked_multiple(buf, protocol, transport, loop):
+async def test_write_payload_chunked_multiple(buf,
+                                              protocol,
+                                              transport, loop) -> None:
     msg = http.StreamWriter(protocol, loop)
     msg.enable_chunking()
     await msg.write(b'data1')
@@ -69,7 +72,7 @@ async def test_write_payload_chunked_multiple(buf, protocol, transport, loop):
     assert b'5\r\ndata1\r\n5\r\ndata2\r\n0\r\n\r\n' == buf
 
 
-async def test_write_payload_length(protocol, transport, loop):
+async def test_write_payload_length(protocol, transport, loop) -> None:
     write = transport.write = mock.Mock()
 
     msg = http.StreamWriter(protocol, loop)
@@ -82,7 +85,7 @@ async def test_write_payload_length(protocol, transport, loop):
     assert b'da' == content.split(b'\r\n\r\n', 1)[-1]
 
 
-async def test_write_payload_chunked_filter(protocol, transport, loop):
+async def test_write_payload_chunked_filter(protocol, transport, loop) -> None:
     write = transport.write = mock.Mock()
 
     msg = http.StreamWriter(protocol, loop)
@@ -118,7 +121,8 @@ compressor = zlib.compressobj(wbits=-zlib.MAX_WBITS)
 COMPRESSED = b''.join([compressor.compress(b'data'), compressor.flush()])
 
 
-async def test_write_payload_deflate_compression(protocol, transport, loop):
+async def test_write_payload_deflate_compression(protocol,
+                                                 transport, loop) -> None:
     write = transport.write = mock.Mock()
     msg = http.StreamWriter(protocol, loop)
     msg.enable_compression('deflate')
@@ -147,7 +151,7 @@ async def test_write_payload_deflate_and_chunked(
     assert b'6\r\nKI,I\x04\x00\r\n0\r\n\r\n' == buf
 
 
-async def test_write_drain(protocol, transport, loop):
+async def test_write_drain(protocol, transport, loop) -> None:
     msg = http.StreamWriter(protocol, loop)
     msg.drain = make_mocked_coro()
     await msg.write(b'1' * (64 * 1024 * 2), drain=False)
@@ -158,7 +162,7 @@ async def test_write_drain(protocol, transport, loop):
     assert msg.buffer_size == 0
 
 
-async def test_write_calls_callback(protocol, transport, loop):
+async def test_write_calls_callback(protocol, transport, loop) -> None:
     on_chunk_sent = make_mocked_coro()
     msg = http.StreamWriter(
         protocol, loop,
@@ -170,7 +174,7 @@ async def test_write_calls_callback(protocol, transport, loop):
     assert on_chunk_sent.call_args == mock.call(chunk)
 
 
-async def test_write_eof_calls_callback(protocol, transport, loop):
+async def test_write_eof_calls_callback(protocol, transport, loop) -> None:
     on_chunk_sent = make_mocked_coro()
     msg = http.StreamWriter(
         protocol, loop,
@@ -182,7 +186,7 @@ async def test_write_eof_calls_callback(protocol, transport, loop):
     assert on_chunk_sent.call_args == mock.call(chunk)
 
 
-async def test_write_to_closing_transport(protocol, transport, loop):
+async def test_write_to_closing_transport(protocol, transport, loop) -> None:
     msg = http.StreamWriter(protocol, loop)
 
     await msg.write(b'Before closing')
@@ -192,13 +196,13 @@ async def test_write_to_closing_transport(protocol, transport, loop):
         await msg.write(b'After closing')
 
 
-async def test_drain(protocol, transport, loop):
+async def test_drain(protocol, transport, loop) -> None:
     msg = http.StreamWriter(protocol, loop)
     await msg.drain()
     assert protocol._drain_helper.called
 
 
-async def test_drain_no_transport(protocol, transport, loop):
+async def test_drain_no_transport(protocol, transport, loop) -> None:
     msg = http.StreamWriter(protocol, loop)
     msg._protocol.transport = None
     await msg.drain()
