@@ -1,14 +1,18 @@
 """Tests for streams.py"""
 
+import abc
 import asyncio
+import gc
 import re
+import types
+from collections import defaultdict
+from itertools import groupby
 from unittest import mock
 
 import pytest
 
 from aiohttp import streams
 
-
 DATA = b'line1\nline2\nline3\n'
 
 
@@ -17,7 +21,8 @@ def chunkify(seq, n):
         yield seq[i:i+n]
 
 
-def create_stream(loop):
+async def create_stream():
+    loop = asyncio.get_event_loop()
     protocol = mock.Mock(_reading_paused=False)
     stream = streams.StreamReader(protocol, loop=loop)
     stream.feed_data(DATA)
@@ -30,6 +35,38 @@ def protocol():
     return mock.Mock(_reading_paused=False)
 
 
+MEMLEAK_SKIP_TYPES = (
+    *(getattr(types, name) for name in types.__all__ if name.endswith('Type')),
+    mock.Mock,
+    abc.ABCMeta,
+)
+
+
+def get_memory_usage(obj):
+    objs = [obj]
+    # Memory leak may be caused by leaked links to same objects.
+    # Without link counting, [1,2,3] is indistiguishable from [1,2,3,3,3,3,3,3]
+    known = defaultdict(int)
+    known[id(obj)] += 1
+
+    while objs:
+        refs = gc.get_referents(*objs)
+        objs = []
+        for obj in refs:
+            if isinstance(obj, MEMLEAK_SKIP_TYPES):
+                continue
+            i = id(obj)
+            known[i] += 1
+            if known[i] == 1:
+                objs.append(obj)
+
+        # Make list of unhashable objects uniq
+        objs.sort(key=id)
+        objs = [next(g) for (i, g) in groupby(objs, id)]
+
+    return sum(known.values())
+
+
 class TestStreamReader:
 
     DATA = b'line1\nline2\nline3\n'
@@ -38,20 +75,21 @@ class TestStreamReader:
         return streams.StreamReader(mock.Mock(_reading_paused=False),
                                     *args, **kwargs)
 
-    async def test_create_waiter(self, loop):
+    async def test_create_waiter(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one(loop=loop)
         stream._waiter = loop.create_future
         with pytest.raises(RuntimeError):
             await stream._wait('test')
 
-    def test_ctor_global_loop(self):
+    def test_ctor_global_loop(self) -> None:
         loop = asyncio.new_event_loop()
         asyncio.set_event_loop(loop)
         stream = streams.StreamReader(mock.Mock(_reading_paused=False))
 
         assert stream._loop is loop
 
-    async def test_at_eof(self):
+    async def test_at_eof(self) -> None:
         stream = self._make_one()
         assert not stream.at_eof()
 
@@ -66,7 +104,8 @@ class TestStreamReader:
         await stream.readline()
         assert stream.at_eof()
 
-    async def test_wait_eof(self, loop):
+    async def test_wait_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
         wait_task = loop.create_task(stream.wait_eof())
 
@@ -79,7 +118,8 @@ class TestStreamReader:
         assert stream.is_eof()
         assert stream._eof_waiter is None
 
-    async def test_wait_eof_eof(self, loop):
+    async def test_wait_eof_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
         stream.feed_eof()
 
@@ -87,7 +127,7 @@ class TestStreamReader:
         await wait_task
         assert stream.is_eof()
 
-    async def test_feed_empty_data(self):
+    async def test_feed_empty_data(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'')
         stream.feed_eof()
@@ -95,7 +135,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'' == data
 
-    async def test_feed_nonempty_data(self):
+    async def test_feed_nonempty_data(self) -> None:
         stream = self._make_one()
         stream.feed_data(self.DATA)
         stream.feed_eof()
@@ -103,7 +143,7 @@ class TestStreamReader:
         data = await stream.read()
         assert self.DATA == data
 
-    async def test_read_zero(self):
+    async def test_read_zero(self) -> None:
         # Read zero bytes.
         stream = self._make_one()
         stream.feed_data(self.DATA)
@@ -115,7 +155,8 @@ class TestStreamReader:
         data = await stream.read()
         assert self.DATA == data
 
-    async def test_read(self, loop):
+    async def test_read(self) -> None:
+        loop = asyncio.get_event_loop()
         # Read bytes.
         stream = self._make_one()
         read_task = loop.create_task(stream.read(30))
@@ -131,7 +172,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'' == data
 
-    async def test_read_line_breaks(self):
+    async def test_read_line_breaks(self) -> None:
         # Read bytes without line breaks.
         stream = self._make_one()
         stream.feed_data(b'line1')
@@ -143,7 +184,7 @@ class TestStreamReader:
         data = await stream.read(5)
         assert b'line2' == data
 
-    async def test_read_all(self):
+    async def test_read_all(self) -> None:
         # Read all available buffered bytes
         stream = self._make_one()
         stream.feed_data(b'line1')
@@ -153,7 +194,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'line1line2' == data
 
-    async def test_read_up_to(self):
+    async def test_read_up_to(self) -> None:
         # Read available buffered bytes up to requested amount
         stream = self._make_one()
         stream.feed_data(b'line1')
@@ -165,7 +206,8 @@ class TestStreamReader:
         data = await stream.read(8)
         assert b'e2' == data
 
-    async def test_read_eof(self, loop):
+    async def test_read_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         # Read bytes, stop at eof.
         stream = self._make_one()
         read_task = loop.create_task(stream.read(1024))
@@ -180,7 +222,7 @@ class TestStreamReader:
         data = await stream.read()
         assert data == b''
 
-    async def test_read_eof_infinit(self):
+    async def test_read_eof_infinite(self) -> None:
         # Read bytes.
         stream = self._make_one()
         stream.feed_eof()
@@ -194,7 +236,7 @@ class TestStreamReader:
             await stream.read()
         assert internal_logger.warning.called
 
-    async def test_read_eof_unread_data_no_warning(self):
+    async def test_read_eof_unread_data_no_warning(self) -> None:
         # Read bytes.
         stream = self._make_one()
         stream.feed_eof()
@@ -205,12 +247,14 @@ class TestStreamReader:
             await stream.read()
             await stream.read()
             await stream.read()
-        stream.unread_data(b'data')
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(b'data')
         await stream.read()
         await stream.read()
         assert not internal_logger.warning.called
 
-    async def test_read_until_eof(self, loop):
+    async def test_read_until_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         # Read all bytes until eof.
         stream = self._make_one()
         read_task = loop.create_task(stream.read(-1))
@@ -227,7 +271,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'' == data
 
-    async def test_read_exception(self):
+    async def test_read_exception(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line\n')
 
@@ -238,7 +282,8 @@ class TestStreamReader:
         with pytest.raises(ValueError):
             await stream.read(2)
 
-    async def test_readline(self, loop):
+    async def test_readline(self) -> None:
+        loop = asyncio.get_event_loop()
         # Read one line. 'readline' will need to wait for the data
         # to come from 'cb'
         stream = self._make_one()
@@ -258,7 +303,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b' chunk4' == data
 
-    async def test_readline_limit_with_existing_data(self):
+    async def test_readline_limit_with_existing_data(self) -> None:
         # Read one line. The data is in StreamReader's buffer
         # before the event loop is run.
 
@@ -273,7 +318,8 @@ class TestStreamReader:
         data = await stream.read()
         assert b'line2\n' == data
 
-    async def test_readline_limit(self, loop):
+    async def test_readline_limit(self) -> None:
+        loop = asyncio.get_event_loop()
         # Read one line. StreamReaders are fed with data after
         # their 'readline' methods are called.
         stream = self._make_one(limit=4)
@@ -290,7 +336,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'chunk3\n' == data
 
-    async def test_readline_nolimit_nowait(self):
+    async def test_readline_nolimit_nowait(self) -> None:
         # All needed data for the first 'readline' call will be
         # in the buffer.
         stream = self._make_one()
@@ -304,7 +350,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'line2\nline3\n' == data
 
-    async def test_readline_eof(self):
+    async def test_readline_eof(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'some data')
         stream.feed_eof()
@@ -312,14 +358,14 @@ class TestStreamReader:
         line = await stream.readline()
         assert b'some data' == line
 
-    async def test_readline_empty_eof(self):
+    async def test_readline_empty_eof(self) -> None:
         stream = self._make_one()
         stream.feed_eof()
 
         line = await stream.readline()
         assert b'' == line
 
-    async def test_readline_read_byte_count(self):
+    async def test_readline_read_byte_count(self) -> None:
         stream = self._make_one()
         stream.feed_data(self.DATA)
 
@@ -332,7 +378,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'ine3\n' == data
 
-    async def test_readline_exception(self):
+    async def test_readline_exception(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line\n')
 
@@ -343,7 +389,7 @@ class TestStreamReader:
         with pytest.raises(ValueError):
             await stream.readline()
 
-    async def test_readexactly_zero_or_less(self):
+    async def test_readexactly_zero_or_less(self) -> None:
         # Read exact number of bytes (zero or less).
         stream = self._make_one()
         stream.feed_data(self.DATA)
@@ -363,7 +409,8 @@ class TestStreamReader:
         data = await stream.read()
         assert self.DATA == data
 
-    async def test_readexactly(self, loop):
+    async def test_readexactly(self) -> None:
+        loop = asyncio.get_event_loop()
         # Read exact number of bytes.
         stream = self._make_one()
 
@@ -383,7 +430,8 @@ class TestStreamReader:
         data = await stream.read()
         assert self.DATA == data
 
-    async def test_readexactly_eof(self, loop):
+    async def test_readexactly_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         # Read exact number of bytes (eof).
         stream = self._make_one(loop=loop)
         n = 2 * len(self.DATA)
@@ -403,7 +451,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'' == data
 
-    async def test_readexactly_exception(self):
+    async def test_readexactly_exception(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line\n')
 
@@ -414,7 +462,7 @@ class TestStreamReader:
         with pytest.raises(ValueError):
             await stream.readexactly(2)
 
-    async def test_unread_data(self):
+    async def test_unread_data(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line1')
         stream.feed_data(b'line2')
@@ -423,7 +471,8 @@ class TestStreamReader:
         data = await stream.read(5)
         assert b'line1' == data
 
-        stream.unread_data(data)
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(data)
 
         data = await stream.read(5)
         assert b'line1' == data
@@ -431,7 +480,8 @@ class TestStreamReader:
         data = await stream.read(4)
         assert b'line' == data
 
-        stream.unread_data(b'line1line')
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(b'line1line')
 
         data = b''
         while len(data) < 10:
@@ -441,23 +491,26 @@ class TestStreamReader:
         data = await stream.read(7)
         assert b'onemore' == data
 
-        stream.unread_data(data)
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(data)
 
         data = b''
         while len(data) < 11:
             data += await stream.read(11)
         assert b'onemoreline' == data
 
-        stream.unread_data(b'line')
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(b'line')
         data = await stream.read(4)
         assert b'line' == data
 
         stream.feed_eof()
-        stream.unread_data(b'at_eof')
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(b'at_eof')
         data = await stream.read(6)
         assert b'at_eof' == data
 
-    async def test_exception(self):
+    async def test_exception(self) -> None:
         stream = self._make_one()
         assert stream.exception() is None
 
@@ -465,7 +518,8 @@ class TestStreamReader:
         stream.set_exception(exc)
         assert stream.exception() is exc
 
-    async def test_exception_waiter(self, loop):
+    async def test_exception_waiter(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
 
         async def set_err():
@@ -478,7 +532,8 @@ class TestStreamReader:
         with pytest.raises(ValueError):
             t1.result()
 
-    async def test_exception_cancel(self, loop):
+    async def test_exception_cancel(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
 
         async def read_a_line():
@@ -493,7 +548,8 @@ class TestStreamReader:
         await asyncio.sleep(0)
         assert stream._waiter is None
 
-    async def test_readany_eof(self, loop):
+    async def test_readany_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
         read_task = loop.create_task(stream.readany())
         loop.call_soon(stream.feed_data, b'chunk1\n')
@@ -504,7 +560,8 @@ class TestStreamReader:
         data = await stream.read()
         assert b'' == data
 
-    async def test_readany_empty_eof(self, loop):
+    async def test_readany_empty_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
         stream.feed_eof()
         read_task = loop.create_task(stream.readany())
@@ -513,7 +570,7 @@ class TestStreamReader:
 
         assert b'' == data
 
-    async def test_readany_exception(self):
+    async def test_readany_exception(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line\n')
 
@@ -524,7 +581,7 @@ class TestStreamReader:
         with pytest.raises(ValueError):
             await stream.readany()
 
-    async def test_read_nowait(self):
+    async def test_read_nowait(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line1\nline2\n')
 
@@ -534,7 +591,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'' == data
 
-    async def test_read_nowait_n(self):
+    async def test_read_nowait_n(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line1\nline2\n')
 
@@ -545,7 +602,7 @@ class TestStreamReader:
         data = await stream.read()
         assert b'' == data
 
-    async def test_read_nowait_exception(self):
+    async def test_read_nowait_exception(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line\n')
         stream.set_exception(ValueError())
@@ -553,7 +610,8 @@ class TestStreamReader:
         with pytest.raises(ValueError):
             stream.read_nowait()
 
-    async def test_read_nowait_waiter(self, loop):
+    async def test_read_nowait_waiter(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
         stream.feed_data(b'line\n')
         stream._waiter = loop.create_future()
@@ -561,7 +619,8 @@ class TestStreamReader:
         with pytest.raises(RuntimeError):
             stream.read_nowait()
 
-    async def test_readchunk(self, loop):
+    async def test_readchunk(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
 
         def cb():
@@ -582,7 +641,8 @@ class TestStreamReader:
         assert b'' == data
         assert not end_of_chunk
 
-    async def test_readchunk_wait_eof(self, loop):
+    async def test_readchunk_wait_eof(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
 
         async def cb():
@@ -595,7 +655,7 @@ class TestStreamReader:
         assert not end_of_chunk
         assert stream.is_eof()
 
-    async def test_begin_and_end_chunk_receiving(self):
+    async def test_begin_and_end_chunk_receiving(self) -> None:
         stream = self._make_one()
 
         stream.begin_http_chunk_receiving()
@@ -626,12 +686,45 @@ class TestStreamReader:
         assert b'' == data
         assert not end_of_chunk
 
-    async def test_end_chunk_receiving_without_begin(self):
+    async def test_readany_chunk_end_race(self) -> None:
+        stream = self._make_one()
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'part1')
+
+        data = await stream.readany()
+        assert data == b'part1'
+
+        loop = asyncio.get_event_loop()
+        task = loop.create_task(stream.readany())
+
+        # Give a chance for task to create waiter and start waiting for it.
+        await asyncio.sleep(0.1)
+        assert stream._waiter is not None
+        assert not task.done()  # Just for sure.
+
+        # This will trigger waiter, but without feeding any data.
+        # The stream should re-create waiter again.
+        stream.end_http_chunk_receiving()
+
+        # Give a chance for task to resolve.
+        # If everything is OK, previous action SHOULD NOT resolve the task.
+        await asyncio.sleep(0.1)
+        assert not task.done()  # The actual test.
+
+        stream.begin_http_chunk_receiving()
+        # This SHOULD unblock the task actually.
+        stream.feed_data(b'part2')
+        stream.end_http_chunk_receiving()
+
+        data = await task
+        assert data == b'part2'
+
+    async def test_end_chunk_receiving_without_begin(self) -> None:
         stream = self._make_one()
         with pytest.raises(RuntimeError):
             stream.end_http_chunk_receiving()
 
-    async def test_readchunk_with_unread(self):
+    async def test_readchunk_with_unread(self) -> None:
         """Test that stream.unread does not break controlled chunk receiving.
         """
         stream = self._make_one()
@@ -648,7 +741,8 @@ class TestStreamReader:
         data, end_of_chunk = await stream.readchunk()
 
         # Try to unread a part of the first chunk
-        stream.unread_data(b'rt1')
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(b'rt1')
 
         # The end_of_chunk signal was already received for the first chunk,
         # so we receive up to the second one
@@ -657,7 +751,8 @@ class TestStreamReader:
         assert end_of_chunk
 
         # Unread a part of the second chunk
-        stream.unread_data(b'rt2')
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(b'rt2')
 
         data, end_of_chunk = await stream.readchunk()
         assert b'rt2' == data
@@ -669,7 +764,7 @@ class TestStreamReader:
         assert b'' == data
         assert not end_of_chunk
 
-    async def test_readchunk_with_other_read_calls(self):
+    async def test_readchunk_with_other_read_calls(self) -> None:
         """Test that stream.readchunk works when other read calls are made on
         the stream.
         """
@@ -681,6 +776,9 @@ class TestStreamReader:
         stream.begin_http_chunk_receiving()
         stream.feed_data(b'part2')
         stream.end_http_chunk_receiving()
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'part3')
+        stream.end_http_chunk_receiving()
 
         data = await stream.read(7)
         assert b'part1pa' == data
@@ -689,57 +787,184 @@ class TestStreamReader:
         assert b'rt2' == data
         assert end_of_chunk
 
+        # Corner case between read/readchunk
+        data = await stream.read(5)
+        assert b'part3' == data
+
+        data, end_of_chunk = await stream.readchunk()
+        assert b'' == data
+        assert end_of_chunk
+
         stream.feed_eof()
+
         data, end_of_chunk = await stream.readchunk()
         assert b'' == data
         assert not end_of_chunk
 
-    async def test___repr__(self):
+    async def test_chunksplits_memory_leak(self) -> None:
+        """ Test for memory leak on chunksplits """
+        stream = self._make_one()
+
+        N = 500
+
+        # Warm-up variables
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'Y' * N)
+        stream.end_http_chunk_receiving()
+        await stream.read(N)
+
+        N = 300
+
+        before = get_memory_usage(stream)
+        for _ in range(N):
+            stream.begin_http_chunk_receiving()
+            stream.feed_data(b'X')
+            stream.end_http_chunk_receiving()
+        await stream.read(N)
+        after = get_memory_usage(stream)
+
+        assert abs(after - before) == 0
+
+    async def test_read_empty_chunks(self) -> None:
+        """Test that feeding empty chunks does not break stream"""
+        stream = self._make_one()
+
+        # Simulate empty first chunk. This is significant special case
+        stream.begin_http_chunk_receiving()
+        stream.end_http_chunk_receiving()
+
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'ungzipped')
+        stream.end_http_chunk_receiving()
+
+        # Possible when compression is enabled.
+        stream.begin_http_chunk_receiving()
+        stream.end_http_chunk_receiving()
+
+        # is also possible
+        stream.begin_http_chunk_receiving()
+        stream.end_http_chunk_receiving()
+
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b' data')
+        stream.end_http_chunk_receiving()
+
+        stream.feed_eof()
+
+        data = await stream.read()
+        assert data == b'ungzipped data'
+
+    async def test_readchunk_separate_http_chunk_tail(self) -> None:
+        """Test that stream.readchunk returns (b'', True) when end of
+        http chunk received after body
+        """
+        loop = asyncio.get_event_loop()
+        stream = self._make_one()
+
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'part1')
+
+        data, end_of_chunk = await stream.readchunk()
+        assert b'part1' == data
+        assert not end_of_chunk
+
+        async def cb():
+            await asyncio.sleep(0.1)
+            stream.end_http_chunk_receiving()
+
+        loop.create_task(cb())
+        data, end_of_chunk = await stream.readchunk()
+        assert b'' == data
+        assert end_of_chunk
+
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'part2')
+        data, end_of_chunk = await stream.readchunk()
+        assert b'part2' == data
+        assert not end_of_chunk
+
+        stream.end_http_chunk_receiving()
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'part3')
+        stream.end_http_chunk_receiving()
+
+        data, end_of_chunk = await stream.readchunk()
+        assert b'' == data
+        assert end_of_chunk
+
+        data, end_of_chunk = await stream.readchunk()
+        assert b'part3' == data
+        assert end_of_chunk
+
+        stream.begin_http_chunk_receiving()
+        stream.feed_data(b'part4')
+        data, end_of_chunk = await stream.readchunk()
+        assert b'part4' == data
+        assert not end_of_chunk
+
+        async def cb():
+            await asyncio.sleep(0.1)
+            stream.end_http_chunk_receiving()
+            stream.feed_eof()
+
+        loop.create_task(cb())
+        data, end_of_chunk = await stream.readchunk()
+        assert b'' == data
+        assert end_of_chunk
+
+        data, end_of_chunk = await stream.readchunk()
+        assert b'' == data
+        assert not end_of_chunk
+
+    async def test___repr__(self) -> None:
         stream = self._make_one()
         assert "<StreamReader>" == repr(stream)
 
-    async def test___repr__nondefault_limit(self):
+    async def test___repr__nondefault_limit(self) -> None:
         stream = self._make_one(limit=123)
         assert "<StreamReader low=123 high=246>" == repr(stream)
 
-    async def test___repr__eof(self):
+    async def test___repr__eof(self) -> None:
         stream = self._make_one()
         stream.feed_eof()
         assert "<StreamReader eof>" == repr(stream)
 
-    async def test___repr__data(self):
+    async def test___repr__data(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'data')
         assert "<StreamReader 4 bytes>" == repr(stream)
 
-    def test___repr__exception(self, loop):
+    async def test___repr__exception(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one(loop=loop)
         exc = RuntimeError()
         stream.set_exception(exc)
         assert "<StreamReader e=RuntimeError()>" == repr(stream)
 
-    async def test___repr__waiter(self, loop):
+    async def test___repr__waiter(self) -> None:
+        loop = asyncio.get_event_loop()
         stream = self._make_one()
         stream._waiter = loop.create_future()
-        assert re.search("<StreamReader w=<Future pending[\S ]*>>",
+        assert re.search(r"<StreamReader w=<Future pending[\S ]*>>",
                          repr(stream))
         stream._waiter.set_result(None)
         await stream._waiter
         stream._waiter = None
         assert "<StreamReader>" == repr(stream)
 
-    async def test_unread_empty(self):
+    async def test_unread_empty(self) -> None:
         stream = self._make_one()
         stream.feed_data(b'line1')
         stream.feed_eof()
-        stream.unread_data(b'')
+        with pytest.warns(DeprecationWarning):
+            stream.unread_data(b'')
 
         data = await stream.read(5)
         assert b'line1' == data
         assert stream.at_eof()
 
 
-async def test_empty_stream_reader():
+async def test_empty_stream_reader() -> None:
     s = streams.EmptyStreamReader()
     assert s.set_exception(ValueError()) is None
     assert s.exception() is None
@@ -750,41 +975,42 @@ async def test_empty_stream_reader():
     assert await s.read() == b''
     assert await s.readline() == b''
     assert await s.readany() == b''
-    assert await s.readchunk() == (b'', False)
+    assert await s.readchunk() == (b'', True)
     with pytest.raises(asyncio.IncompleteReadError):
         await s.readexactly(10)
     assert s.read_nowait() == b''
 
 
 @pytest.fixture
-def buffer(loop):
-    return streams.DataQueue(loop=loop)
+async def buffer(loop):
+    return streams.DataQueue(loop)
 
 
 class TestDataQueue:
 
-    def test_is_eof(self, buffer):
+    def test_is_eof(self, buffer) -> None:
         assert not buffer.is_eof()
         buffer.feed_eof()
         assert buffer.is_eof()
 
-    def test_at_eof(self, buffer):
+    def test_at_eof(self, buffer) -> None:
         assert not buffer.at_eof()
         buffer.feed_eof()
         assert buffer.at_eof()
         buffer._buffer.append(object())
         assert not buffer.at_eof()
 
-    def test_feed_data(self, buffer):
+    def test_feed_data(self, buffer) -> None:
         item = object()
         buffer.feed_data(item, 1)
         assert [(item, 1)] == list(buffer._buffer)
 
-    def test_feed_eof(self, buffer):
+    def test_feed_eof(self, buffer) -> None:
         buffer.feed_eof()
         assert buffer._eof
 
-    async def test_read(self, buffer, loop):
+    async def test_read(self, buffer) -> None:
+        loop = asyncio.get_event_loop()
         item = object()
 
         def cb():
@@ -794,7 +1020,9 @@ class TestDataQueue:
         data = await buffer.read()
         assert item is data
 
-    async def test_read_eof(self, buffer, loop):
+    async def test_read_eof(self, buffer) -> None:
+        loop = asyncio.get_event_loop()
+
         def cb():
             buffer.feed_eof()
         loop.call_soon(cb)
@@ -802,7 +1030,8 @@ class TestDataQueue:
         with pytest.raises(streams.EofStream):
             await buffer.read()
 
-    async def test_read_cancelled(self, buffer, loop):
+    async def test_read_cancelled(self, buffer) -> None:
+        loop = asyncio.get_event_loop()
         read_task = loop.create_task(buffer.read())
         await asyncio.sleep(0)
         waiter = buffer._waiter
@@ -817,7 +1046,7 @@ class TestDataQueue:
         buffer.feed_data(b'test', 4)
         assert buffer._waiter is None
 
-    async def test_read_until_eof(self, buffer):
+    async def test_read_until_eof(self, buffer) -> None:
         item = object()
         buffer.feed_data(item, 1)
         buffer.feed_eof()
@@ -828,7 +1057,7 @@ class TestDataQueue:
         with pytest.raises(streams.EofStream):
             await buffer.read()
 
-    async def test_read_exc(self, buffer):
+    async def test_read_exc(self, buffer) -> None:
         item = object()
         buffer.feed_data(item)
         buffer.set_exception(ValueError)
@@ -839,13 +1068,13 @@ class TestDataQueue:
         with pytest.raises(ValueError):
             await buffer.read()
 
-    async def test_read_exception(self, buffer):
+    async def test_read_exception(self, buffer) -> None:
         buffer.set_exception(ValueError())
 
         with pytest.raises(ValueError):
             await buffer.read()
 
-    async def test_read_exception_with_data(self, buffer):
+    async def test_read_exception_with_data(self, buffer) -> None:
         val = object()
         buffer.feed_data(val, 1)
         buffer.set_exception(ValueError())
@@ -854,7 +1083,8 @@ class TestDataQueue:
         with pytest.raises(ValueError):
             await buffer.read()
 
-    async def test_read_exception_on_wait(self, buffer, loop):
+    async def test_read_exception_on_wait(self, buffer) -> None:
+        loop = asyncio.get_event_loop()
         read_task = loop.create_task(buffer.read())
         await asyncio.sleep(0)
         assert asyncio.isfuture(buffer._waiter)
@@ -865,14 +1095,15 @@ class TestDataQueue:
         with pytest.raises(ValueError):
             await read_task
 
-    def test_exception(self, buffer):
+    def test_exception(self, buffer) -> None:
         assert buffer.exception() is None
 
         exc = ValueError()
         buffer.set_exception(exc)
         assert buffer.exception() is exc
 
-    async def test_exception_waiter(self, buffer, loop):
+    async def test_exception_waiter(self, buffer) -> None:
+        loop = asyncio.get_event_loop()
 
         async def set_err():
             buffer.set_exception(ValueError())
@@ -886,7 +1117,8 @@ class TestDataQueue:
             t1.result()
 
 
-def test_feed_data_waiters(loop, protocol):
+async def test_feed_data_waiters(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
@@ -902,7 +1134,8 @@ def test_feed_data_waiters(loop, protocol):
     assert reader._eof_waiter is eof_waiter
 
 
-def test_feed_data_completed_waiters(loop, protocol):
+async def test_feed_data_completed_waiters(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     waiter = reader._waiter = loop.create_future()
 
@@ -912,7 +1145,8 @@ def test_feed_data_completed_waiters(loop, protocol):
     assert reader._waiter is None
 
 
-def test_feed_eof_waiters(loop, protocol):
+async def test_feed_eof_waiters(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
@@ -926,7 +1160,8 @@ def test_feed_eof_waiters(loop, protocol):
     assert reader._eof_waiter is None
 
 
-def test_feed_eof_cancelled(loop, protocol):
+async def test_feed_eof_cancelled(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
@@ -942,7 +1177,8 @@ def test_feed_eof_cancelled(loop, protocol):
     assert reader._eof_waiter is None
 
 
-def test_on_eof(loop, protocol):
+async def test_on_eof(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
 
     on_eof = mock.Mock()
@@ -953,7 +1189,7 @@ def test_on_eof(loop, protocol):
     assert on_eof.called
 
 
-def test_on_eof_empty_reader(loop):
+async def test_on_eof_empty_reader() -> None:
     reader = streams.EmptyStreamReader()
 
     on_eof = mock.Mock()
@@ -962,7 +1198,8 @@ def test_on_eof_empty_reader(loop):
     assert on_eof.called
 
 
-def test_on_eof_exc_in_callback(loop, protocol):
+async def test_on_eof_exc_in_callback(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
 
     on_eof = mock.Mock()
@@ -975,7 +1212,7 @@ def test_on_eof_exc_in_callback(loop, protocol):
     assert not reader._eof_callbacks
 
 
-def test_on_eof_exc_in_callback_empty_stream_reader(loop):
+async def test_on_eof_exc_in_callback_empty_stream_reader() -> None:
     reader = streams.EmptyStreamReader()
 
     on_eof = mock.Mock()
@@ -985,7 +1222,8 @@ def test_on_eof_exc_in_callback_empty_stream_reader(loop):
     assert on_eof.called
 
 
-def test_on_eof_eof_is_set(loop, protocol):
+async def test_on_eof_eof_is_set(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     reader.feed_eof()
 
@@ -995,7 +1233,8 @@ def test_on_eof_eof_is_set(loop, protocol):
     assert not reader._eof_callbacks
 
 
-def test_on_eof_eof_is_set_exception(loop, protocol):
+async def test_on_eof_eof_is_set_exception(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     reader.feed_eof()
 
@@ -1007,7 +1246,8 @@ def test_on_eof_eof_is_set_exception(loop, protocol):
     assert not reader._eof_callbacks
 
 
-def test_set_exception(loop, protocol):
+async def test_set_exception(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
@@ -1021,7 +1261,8 @@ def test_set_exception(loop, protocol):
     assert reader._eof_waiter is None
 
 
-def test_set_exception_cancelled(loop, protocol):
+async def test_set_exception_cancelled(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
@@ -1038,7 +1279,8 @@ def test_set_exception_cancelled(loop, protocol):
     assert reader._eof_waiter is None
 
 
-def test_set_exception_eof_callbacks(loop, protocol):
+async def test_set_exception_eof_callbacks(protocol) -> None:
+    loop = asyncio.get_event_loop()
     reader = streams.StreamReader(protocol, loop=loop)
 
     on_eof = mock.Mock()
@@ -1049,43 +1291,45 @@ def test_set_exception_eof_callbacks(loop, protocol):
     assert not reader._eof_callbacks
 
 
-async def test_stream_reader_lines(loop):
+async def test_stream_reader_lines() -> None:
     line_iter = iter(DATA.splitlines(keepends=True))
-    async for line in create_stream(loop):
+    async for line in await create_stream():
         assert line == next(line_iter, None)
     pytest.raises(StopIteration, next, line_iter)
 
 
-async def test_stream_reader_chunks_complete(loop):
+async def test_stream_reader_chunks_complete() -> None:
     """Tests if chunked iteration works if the chunking works out
     (i.e. the data is divisible by the chunk size)
     """
     chunk_iter = chunkify(DATA, 9)
-    async for data in create_stream(loop).iter_chunked(9):
+    async for data in (await create_stream()).iter_chunked(9):
         assert data == next(chunk_iter, None)
     pytest.raises(StopIteration, next, chunk_iter)
 
 
-async def test_stream_reader_chunks_incomplete(loop):
+async def test_stream_reader_chunks_incomplete() -> None:
     """Tests if chunked iteration works if the last chunk is incomplete"""
     chunk_iter = chunkify(DATA, 8)
-    async for data in create_stream(loop).iter_chunked(8):
+    async for data in (await create_stream()).iter_chunked(8):
         assert data == next(chunk_iter, None)
     pytest.raises(StopIteration, next, chunk_iter)
 
 
-async def test_data_queue_empty(loop):
+async def test_data_queue_empty() -> None:
     """Tests that async looping yields nothing if nothing is there"""
-    buffer = streams.DataQueue(loop=loop)
+    loop = asyncio.get_event_loop()
+    buffer = streams.DataQueue(loop)
     buffer.feed_eof()
 
     async for _ in buffer:  # NOQA
         assert False
 
 
-async def test_data_queue_items(loop):
+async def test_data_queue_items() -> None:
     """Tests that async looping yields objects identically"""
-    buffer = streams.DataQueue(loop=loop)
+    loop = asyncio.get_event_loop()
+    buffer = streams.DataQueue(loop)
 
     items = [object(), object()]
     buffer.feed_data(items[0], 1)
@@ -1098,30 +1342,31 @@ async def test_data_queue_items(loop):
     pytest.raises(StopIteration, next, item_iter)
 
 
-async def test_stream_reader_iter_any(loop):
+async def test_stream_reader_iter_any() -> None:
     it = iter([b'line1\nline2\nline3\n'])
-    async for raw in create_stream(loop).iter_any():
+    async for raw in (await create_stream()).iter_any():
         assert raw == next(it)
     pytest.raises(StopIteration, next, it)
 
 
-async def test_stream_reader_iter(loop):
+async def test_stream_reader_iter() -> None:
     it = iter([b'line1\n',
                b'line2\n',
                b'line3\n'])
-    async for raw in create_stream(loop):
+    async for raw in await create_stream():
         assert raw == next(it)
     pytest.raises(StopIteration, next, it)
 
 
-async def test_stream_reader_iter_chunks_no_chunked_encoding(loop):
+async def test_stream_reader_iter_chunks_no_chunked_encoding() -> None:
     it = iter([b'line1\nline2\nline3\n'])
-    async for data, end_of_chunk in create_stream(loop).iter_chunks():
+    async for data, end_of_chunk in (await create_stream()).iter_chunks():
         assert (data, end_of_chunk) == (next(it), False)
     pytest.raises(StopIteration, next, it)
 
 
-async def test_stream_reader_iter_chunks_chunked_encoding(loop, protocol):
+async def test_stream_reader_iter_chunks_chunked_encoding(protocol) -> None:
+    loop = asyncio.get_event_loop()
     stream = streams.StreamReader(protocol, loop=loop)
     for line in DATA.splitlines(keepends=True):
         stream.begin_http_chunk_receiving()
