@@ -15,7 +15,7 @@ def buf():
 
 
 @pytest.fixture
-def request(buf):
+def http_request(buf):
     method = 'GET'
     path = '/'
     writer = mock.Mock()
@@ -44,7 +44,7 @@ def request(buf):
     return req
 
 
-def test_all_http_exceptions_exported():
+def test_all_http_exceptions_exported() -> None:
     assert 'HTTPException' in web.__all__
     for name in dir(web):
         if name.startswith('_'):
@@ -54,9 +54,9 @@ def test_all_http_exceptions_exported():
             assert name in web.__all__
 
 
-async def test_HTTPOk(buf, request):
+async def test_HTTPOk(buf, http_request) -> None:
     resp = web.HTTPOk()
-    await resp.prepare(request)
+    await resp.prepare(http_request)
     await resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match(('HTTP/1.1 200 OK\r\n'
@@ -67,7 +67,7 @@ async def test_HTTPOk(buf, request):
                      '200: OK'), txt)
 
 
-def test_terminal_classes_has_status_code():
+def test_terminal_classes_has_status_code() -> None:
     terminals = set()
     for name in dir(web):
         obj = getattr(web, name)
@@ -87,11 +87,11 @@ def test_terminal_classes_has_status_code():
     assert 1 == codes.most_common(1)[0][1]
 
 
-async def test_HTTPFound(buf, request):
+async def test_HTTPFound(buf, http_request) -> None:
     resp = web.HTTPFound(location='/redirect')
     assert '/redirect' == resp.location
     assert '/redirect' == resp.headers['location']
-    await resp.prepare(request)
+    await resp.prepare(http_request)
     await resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match('HTTP/1.1 302 Found\r\n'
@@ -103,7 +103,7 @@ async def test_HTTPFound(buf, request):
                     '302: Found', txt)
 
 
-def test_HTTPFound_empty_location():
+def test_HTTPFound_empty_location() -> None:
     with pytest.raises(ValueError):
         web.HTTPFound(location='')
 
@@ -111,12 +111,17 @@ def test_HTTPFound_empty_location():
         web.HTTPFound(location=None)
 
 
-async def test_HTTPMethodNotAllowed(buf, request):
+def test_HTTPFound_location_CRLF() -> None:
+    exc = web.HTTPFound(location='/redirect\r\n')
+    assert '\r\n' not in exc.headers['Location']
+
+
+async def test_HTTPMethodNotAllowed(buf, http_request) -> None:
     resp = web.HTTPMethodNotAllowed('get', ['POST', 'PUT'])
     assert 'GET' == resp.method
-    assert ['POST', 'PUT'] == resp.allowed_methods
+    assert {'POST', 'PUT'} == resp.allowed_methods
     assert 'POST,PUT' == resp.headers['allow']
-    await resp.prepare(request)
+    await resp.prepare(http_request)
     await resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match('HTTP/1.1 405 Method Not Allowed\r\n'
@@ -128,7 +133,7 @@ async def test_HTTPMethodNotAllowed(buf, request):
                     '405: Method Not Allowed', txt)
 
 
-def test_override_body_with_text():
+def test_override_body_with_text() -> None:
     resp = web.HTTPNotFound(text="Page not found")
     assert 404 == resp.status
     assert "Page not found".encode('utf-8') == resp.body
@@ -137,10 +142,11 @@ def test_override_body_with_text():
     assert "utf-8" == resp.charset
 
 
-def test_override_body_with_binary():
+def test_override_body_with_binary() -> None:
     txt = "<html><body>Page not found</body></html>"
-    resp = web.HTTPNotFound(body=txt.encode('utf-8'),
-                            content_type="text/html")
+    with pytest.warns(DeprecationWarning):
+        resp = web.HTTPNotFound(body=txt.encode('utf-8'),
+                                content_type="text/html")
     assert 404 == resp.status
     assert txt.encode('utf-8') == resp.body
     assert txt == resp.text
@@ -148,34 +154,34 @@ def test_override_body_with_binary():
     assert resp.charset is None
 
 
-def test_default_body():
+def test_default_body() -> None:
     resp = web.HTTPOk()
     assert b'200: OK' == resp.body
 
 
-def test_empty_body_204():
+def test_empty_body_204() -> None:
     resp = web.HTTPNoContent()
     assert resp.body is None
 
 
-def test_empty_body_205():
+def test_empty_body_205() -> None:
     resp = web.HTTPNoContent()
     assert resp.body is None
 
 
-def test_empty_body_304():
+def test_empty_body_304() -> None:
     resp = web.HTTPNoContent()
     resp.body is None
 
 
-def test_link_header_451(buf, request):
+def test_link_header_451(buf) -> None:
     resp = web.HTTPUnavailableForLegalReasons(link='http://warning.or.kr/')
 
     assert 'http://warning.or.kr/' == resp.link
     assert '<http://warning.or.kr/>; rel="blocked-by"' == resp.headers['Link']
 
 
-def test_HTTPException_retains_cause():
+def test_HTTPException_retains_cause() -> None:
     with pytest.raises(web.HTTPException) as ei:
         try:
             raise Exception('CustomException')
