@@ -5,7 +5,7 @@ set -euo pipefail
 package_name="$1"
 if [ -z "$package_name" ]
 then
-    &>2 echo "Please pass package name as a first argument of this script ($0)"
+    >&2 echo "Please pass package name as a first argument of this script ($0)"
     exit 1
 fi
 
